<?php

namespace Adherence\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="trade_sales")
 */
class TradeSales
{

  /**
   * Can archive records.
   */
  const ARCHIVE = false;

  /**
   * Pull Synchronization Strategy for this table.
   */
  const PULL_SYNCH_STRATEGY = false;

  /**
   * Push Synchronization Strategy for this table.
   */
  const PUSH_SYNCH_STRATEGY = false;

  /**
   * @ORM\Id
   * @ORM\Column(type="integer");
   * @ORM\GeneratedValue(strategy="AUTO")
   */
  protected $id;

  /**
   * @ORM\Column(type="string", length=128, name="dealer_name")
   */
  protected $dealerName;

  /**
   * @ORM\Column(type="string", length=45, name="stock_number")
   */
  protected $stockNumber;

  /**
   * @ORM\Column(type="string", length=45, name="invoice_number")
   */
  protected $invoiceNumber;

  /**
   * @ORM\Column(type="string", length=45, name="reg_number")
   */
  protected $regNumber;

  /**
   * @ORM\Column(type="string", length=45, name="vin_number")
   */
  protected $vinNumber;

  /**
   * @ORM\Column(type="string", length=45, name="vehicle_make")
   */
  protected $vehicleMake;

  /**
   * @ORM\Column(type="datetime", nullable=true, name="reg_date");
   */
  protected $regDate;

  /**
   * @ORM\Column(type="datetime", nullable=true, name="sale_date");
   */
  protected $saleDate;

  /**
   * @ORM\Column(type="string", length=45, name="sale_type")
   */
  protected $saleType;

  /**
   * @ORM\Column(type="decimal", scale=4, precision=13, nullable=false, name="total_cost", options={"unsigned"=false});
   */
  protected $totalCost;

  /**
   * @ORM\Column(type="decimal", scale=4, precision=13, nullable=false, name="sales_total", options={"unsigned"=false});
   */
  protected $salesTotal;

  /**
   * @ORM\Column(type="string", length=128, name="sold_to")
   */
  protected $soldTo;

  /**
   * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock")
   * @ORM\JoinColumn(nullable=true, name="stock_id")
   * */
  protected $stock;

  /**
   * @ORM\Column(nullable=true, type="boolean", name="vin_match");
   */
  protected $vinMatch;

  /**
   * @ORM\Column(nullable=true, type="boolean", name="reg_match");
   */
  protected $regMatch;

  /**
   * Initialize collections.
   */
  public function __construct()
  {
    
  }

  /**
   * Magic getter to expose protected properties.
   * @param string $property
   * @return mixed
   */
  public function __get($property)
  {
    return $this->$property;
  }

  /**
   * Magic setter to save protected properties.
   * @param string $property
   * @param mixed $value
   */
  public function __set($property, $value)
  {
    $this->$property = $value;
  }

  public function fromArray($data)
  {
    isset($data['dealerName']) && $this->dealerName = $data['dealerName'];
    isset($data['stockNumber']) && $this->stockNumber = $data['stockNumber'];
    isset($data['invoiceNumber']) && $this->invoiceNumber = $data['invoiceNumber'];
    isset($data['regNumber']) && $this->regNumber = $data['regNumber'];
    isset($data['vinNumber']) && $this->vinNumber = $data['vinNumber'];
    isset($data['vehicleMake']) && $this->vehicleMake = $data['vehicleMake'];
    isset($data['regDate']) && $this->regDate = $data['regDate'];
    isset($data['saleDate']) && $this->saleDate = $data['saleDate'];
    isset($data['saleType']) && $this->saleType = $data['saleType'];
    isset($data['totalCost']) && $this->totalCost = $data['totalCost'];
    isset($data['salesTotal']) && $this->salesTotal = $data['salesTotal'];
    isset($data['soldTo']) && $this->soldTo = $data['soldTo'];
    isset($data['stock']) && $this->stock = $data['stock'];
    isset($data['vinMatch']) && $this->vinMatch = $data['vinMatch'];
    isset($data['regMatch']) && $this->regMatch = $data['regMatch'];
  }

  public function toArray()
  {
    return array(
      'id' => $this->id,
      'dealerName' => $this->dealerName,
      'stockNumber' => $this->stockNumber,
      'invoiceNumber' => $this->invoiceNumber,
      'regNumber' => $this->regNumber,
      'vinNumber' => $this->vinNumber,
      'vehicleMake' => $this->vehicleMake,
      'regDate' => $this->regDate,
      'saleDate' => $this->saleDate,
      'saleType' => $this->saleType,
      'totalCost' => $this->totalCost,
      'salesTotal' => $this->salesTotal,
      'soldTo' => $this->soldTo,
      'stock' => $this->stock,
      'vinMatch' => $this->vinMatch,
      'regMatch' => $this->regMatch
    );
  }

}
