<?php
namespace Company\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * Found Method.
 * @ORM\Entity
 * @ORM\Table(name="company_found_method")
 */
class FoundMethod
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=100)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;

	/**
	 * @ORM\OneToMany(targetEntity="FoundMethodDetail", mappedBy="foundMethod", fetch="EXTRA_LAZY")
	 **/
	private $methodDetails;


	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->methodDetails = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Add detail entry for this method.
	 * @param Town $town
	 * @return \Location\Entity\Region
	 */
	public function addDetail(FoundMethodDetail $methodDetail)
	{
		$this->methodDetails[] = $methodDetail;
		return $this;
	}

	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array(), array $intersect = array())
	{
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['name']))
			&& $data['name'] = $this->name;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['name'])
			&& $this->name = $data['name'];
	}

}
