<?php
namespace Company\Service;



/**
 * Manage Group data.
 * @author andre.fourie
 */
class Group extends \Company\DataBin\Group
{


	/**
	 * Contract for custom select list.
	 * @param object|null $jobRecord
	 * @param array $input
	 * @return \Workspace\Contract\Recurring
	 */
	public function contractJoinedSelectList($jobRecord, array $input = array())
	{

		$options = new \Workspace\UseCase\Options();
		$requirement = new \Workspace\UseCase\Requirement();
		$requirement->addRequiredInput(array(
				'Filter' => array(
						'region' 			=> 'Id'
				)
		));
		return new \Workspace\Contract\Recurring($options, $requirement);
	}

	/**
	 * A custom select list.
	 * @param object|null $jobRecord
	 * @param \Workspace\Utility\ServiceInputParams $contract
	 * @return array
	 */
	public function executeJoinedSelectList($jobRecord, \Workspace\Utility\ServiceInputParams $contract)
	{
		$items = $this->em->createQuery(
				'SELECT partial companyGroup.{id,name} '
				. 'FROM Company\Entity\Group companyGroup '
				. 'JOIN companyGroup.divisions divisions '
				. 'JOIN divisions.companies company '
				. 'WHERE IDENTITY(company.region) = :regionId '
				. 'ORDER BY companyGroup.name ASC'
			)
			->setParameter('regionId', $contract->data->Filter['region'])
			->getArrayResult();
		$data = array();
		$ids = array();
		foreach ($items as $item)
		{
			if (isset($ids[$item['id']]))
			{
				continue;
			}
			$ids[$item['id']] = true;
			$data[] = array(
					'value' => $item['id'],
					'label' => $item['name']
			);
		}
		return $contract->success(
				'List retrieved.',
				$data
				);
	}


}