<?php
namespace Location\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A country, in such a small file, imagine that.
 * @ORM\Entity
 * @ORM\Table(name="lib_country", uniqueConstraints={@ORM\UniqueConstraint(name="unique_country", columns={"name"})})
 */
class Country
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Build';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Build';


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=50, unique=true)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="string", length=2, nullable=true, name="country_code_iso")
	 */
	protected $isoCode;

	/**
	 * @ORM\Column(type="string", length=2, nullable=true, name="country_code_fips")
	 */
	protected $fipsCode;

	/**
	 * @ORM\Column(type="smallint", nullable=true, name="dialing_code")
	 */
	protected $dailingCode;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;

	/**
	 * @ORM\OneToMany(targetEntity="Region", mappedBy="country", fetch="EXTRA_LAZY")
	 **/
	private $regions;


	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->regions = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Add a new Region to this Country.
	 * @param Region $region
	 * @return \Location\Entity\Country
	 */
	public function addRegion(Region $region)
	{
		$this->regions[] = $region;
		return $this;
	}

	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array(), array $intersect = array())
	{
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['name']))
			&& $data['name'] = $this->name;
		($includeAll || isset($intersect['isoCode']))
			&& $data['isoCode'] = $this->isoCode;
		($includeAll || isset($intersect['fipsCode']))
			&& $data['fipsCode'] = $this->fipsCode;
		($includeAll || isset($intersect['dailingCode']))
			&& $data['dailingCode'] = $this->dailingCode;
		return $data;
	}

	/**
	 * Convert the object to an array for synchronization.
	 * @return array
	 */
	public function toSynchArray()
	{
		return array(
				'id'   => $this->id,
				'name' => $this->name
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['name'])
			&& $this->name  = $data['name'];
		isset($data['isoCode'])
			&& $this->isoCode  = $data['isoCode'];
		isset($data['fipsCode'])
			&& $this->fipsCode  = $data['fipsCode'];
		isset($data['dailingCode'])
			&& $this->dailingCode  = $data['dailingCode'];
	}

}
