<?php
namespace Newsletter\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A newsletter.
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="newsletter")
 */
class Newsletter
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=100)
	 */
	protected $subject;

	/**
	 * @ORM\Column(type="text")
	 */
	protected $content;

	/**
	 * @ORM\Column(type="integer", name="sent_to")
	 */
	protected $sentTo = 0;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Attachment")
	 * @ORM\JoinColumn(nullable=true, name="lib_attachment_id")
	 **/
	protected $attachment;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="previous_status");
	 */
	protected $previousState;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="status");
	 */
	protected $jobState;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @return array
	 */
	public function toArray(array $expand = array('attachment'))
	{
		return array(
				'id'         => $this->id,
				'subject'    => $this->subject,
				'content'    => $this->content,
				'sentTo'     => $this->sentTo,
				'attachment' => in_array('attachment', $expand)
										&& !is_null($this->attachment)
					? $this->attachment->toArray($expand)
					: null,
				'jobState'   => $this->jobState,
				'created'    => !is_null($this->created)
					? $this->created->format(\Utility\Registry::getConfigParam('DateTimeFormat'))
					: null,
				'updated'    => !is_null($this->updated)
					? $this->updated->format(\Utility\Registry::getConfigParam('DateTimeFormat'))
					: null
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['subject'])
			&& $this->subject  = $data['subject'];
		isset($data['content'])
			&& $this->content  = $data['content'];
		isset($data['sentTo'])
			&& $this->sentTo  = $data['sentTo'];
		isset($data['attachment'])
			&& $this->attachment  = $data['attachment'];
	}

}
