<?php
namespace PriceGuide\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A Price Guide item.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="price_guide")
 */
class PriceGuide
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="created_by_profile_id")
	 **/
	protected $createdBy;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\Valuation\Entity\Valuation")
	 * @ORM\JoinColumn(nullable=true, name="stock_valuation_id")
	 **/
	protected $valuation;

	/**
	 * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock")
	 * @ORM\JoinColumn(nullable=false, name="stock_id")
	 **/
	protected $stock;


	/* ------------------------------------ Linkage ------------------------------------ */
	/**
	 * @ORM\OneToMany(targetEntity="ClubStock", mappedBy="priceGuideStock", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $clubs;

	/**
	 * @ORM\OneToMany(targetEntity="MemberStock", mappedBy="priceGuideStock", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $members;

	/**
	 * @ORM\OneToMany(targetEntity="Offer", mappedBy="priceGuideStock", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $offers;


	/* ------------------------------------ Tracking ------------------------------------ */
	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="previous_status");
	 */
	protected $previousState;

	/**
	 * @ORM\Column(nullable=true, type="string", length=25, name="status");
	 */
	protected $jobState = 'Open4Offers';

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->clubs  = new \Doctrine\Common\Collections\ArrayCollection();
		$this->offers = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Add a new ClubStock to this Stock Item.
	 * @param Damage $damage
	 * @return ClubStock
	 */
	public function addClubStock(ClubStock $clubStock)
	{
		$this->clubs[] = $clubStock;
		return $this;
	}



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->company   = \Utility\Registry::resolveCompanyContext($this->company);
		$this->createdBy = \Utility\Registry::resolveProfileContext($this->createdBy);
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['stock']))
			&& $data['stock'] = (in_array('stock', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->stock)
				? (!$showIdentifiers ? $this->stock->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->stock->id)
				: null;
		($includeAll || isset($intersect['clubs']))
			&& $data['clubs'] = (in_array('clubs', $expand) || $expandAll || $showIdentifiers)
										&& !is_null($this->clubs)
				? $this->clubsToArray($showIdentifiers)
				: null;
		(($includeAll || isset($intersect['clubs'])) && in_array('club-ids', $expand))
			&& $data['clubs'] = (in_array('club-ids', $expand) || $expandAll || $showIdentifiers)
										&& !is_null($this->clubs)
				? $this->clubsToArray(true)
				: null;
		($includeAll || isset($intersect['offers']))
			&& $data['offers'] = (in_array('offers', $expand) || $expandAll || $showIdentifiers)
										&& !is_null($this->offers)
				? $this->offersToArray($showIdentifiers)
				: null;
		($includeAll || isset($intersect['jobState']))
			&& $data['jobState'] = $this->jobState;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Internal utility to change clubs collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function clubsToArray($showIdentifiers = false)
	{
		$data = array();
		$iterator = $this->clubs->getIterator();
		foreach ($iterator as $club)
		{
			$data[] = $showIdentifiers
				? array('id' => $club->id)
				: $club->toArray();
		}
		return $data;
	}

	/**
	 * Internal utility to change stock offers collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function offersToArray($showIdentifiers = false)
	{
		$data = array();
		$iterator = $this->offers->getIterator();
		foreach ($iterator as $offer)
		{
			$data[] = $showIdentifiers
				? array('id' => $offer->id)
				: $offer->toArray();
		}
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['company'])
			&& $this->company = $data['company'];
		isset($data['valuation'])
			&& $this->valuation = $data['valuation'];
		isset($data['stock'])
			&& $this->stock = $data['stock'];
		if (isset($data['clubs']))
		{
			$em = \Utility\Registry::getEntityManager();
			foreach ($data['clubs'] as $cl)
			{
				if (!is_numeric($cl['id']))
				{
					continue;
				}
				$clubStock = new \PriceGuide\Entity\ClubStock();
				$clubStock->club = $em->getReference('\PriceGuide\Entity\Club', $cl['id']);
				if (is_null($clubStock->club))
				{
					continue;
				}
				$clubStock->priceGuideStock = $this;
				$em->persist($clubStock);
				$this->clubs->add($clubStock);
			}
		}
	}



}

