<?php
namespace Report\Report;



/**
 * Stock Control Report.
 * @author andre.fourie
 */
class StockAuction extends \Utility\Service\Report
{



	/**
	 * @var string
	 */
	protected $_title  = 'Stock Auction Report';
	/**
	 * @var string
	 */
	protected $_subject = 'Stock Auction';
	/**
	 * @var string
	 */
	protected $_description = 'Historical report for vehicles sold on auction.';
	/**
	 * @var array
	 */
	protected $_notes = array(
			'Confidential information, generated using Bid 4 Cars, for more information visit bid4cars.com.na'
	);
	/**
	 * @var array
	 */
	protected $_headers = array(
			'Auction ID',
			'Buyer Dealership',
			'Vehicle Make',
			'Type',
			'Year',
			'Stock No.',
			'Registration No.',
            'Kilometres',
			'Date Loaded',
			'Date Sold',
			'Trade Price',
			'Reserve Price',
			'Total Recon',
			'Highest Bid'
	);
	/**
	 * @var array
	 */
	protected $_fields = array(
			'auction_id',
			'buyerCompany_name',
			'make_name',
			'type_name',
			'year_name',
			'stock_stockNumber',
			'stock_registrationNumber',
            'stock_km',
			'auction_created',
			'auction_endDate',
			'stock_tradePrice',
			'auction_reservePrice',
            'stock_damageTotal',
			'auction_currentBidPrice',
	);
	/**
	 * @var array
	 */
	protected $_totals = array(
			'auction_currentBidPrice',
			'stock_tradePrice',
			'auction_reservePrice',
            'stock_damageTotal',
	);
	/**
	 * @var array
	 */
	protected $_currencyFields = array(
			'auction_currentBidPrice',
			'stock_tradePrice',
			'auction_reservePrice',
			'stock_damageTotal'
	);



	/**
	 * Build the dataset.
	 */
	public function build()
	{
    ini_set('memory_limit','1024M');
		#-> Prepare parameters.
		$selection = array(
				'auction', 'stock', 'type', 'year', 'make',
				'buyerCompany', 'buyerGroup', 'buyerDivision'
		);
		$join      = array();
		$leftJion  = array();
		$where     = array();
		$params    = array();
		$query = 'SELECT [SELECTION] '
				. 'FROM \Auction\Entity\Auction auction '
				. ' JOIN auction.stock stock '
				. ' JOIN stock.company sellerCompany '
				. ' JOIN auction.bids bid '
				. ' LEFT JOIN stock.type type '
				. ' LEFT JOIN stock.vehicleYear year '
				. ' LEFT JOIN type.model model '
				. ' LEFT JOIN model.make make '
				. ' LEFT JOIN auction.createdBy sellerProfile '
				. ' LEFT JOIN auction.currentBid currentBid '
				. ' LEFT JOIN currentBid.profile buyerProfile '
				. ' LEFT JOIN currentBid.company buyerCompany '
				. ' LEFT JOIN sellerCompany.group sellerGroup '
				. ' LEFT JOIN sellerCompany.groupDivision sellerDivision '
				. ' LEFT JOIN buyerProfile.group buyerGroup '
				. ' LEFT JOIN buyerProfile.groupDivision buyerDivision '
				. '[WHERE] '
				. 'GROUP BY auction.id '
				. 'ORDER BY auction.created ASC';
		$where[] = 'auction.jobState = \'Sold\'';

		#-> Construct details.
		$authData = \Utility\Registry::getAuthData();
		if (isset($this->_input['dateFrom']) && !empty($this->_input['dateFrom']))
		{
			$this->_queries['Date Range From'] = $this->_input['dateFrom'];
			$where[] = 'auction.endDate >= :dateFrom';
			$params['dateFrom'] = new \DateTime($this->_input['dateFrom']);
		}
		if (isset($this->_input['dateTo']) && !empty($this->_input['dateTo']))
		{
			$this->_queries['Date Range Until'] = $this->_input['dateTo'];
			$where[] = 'auction.endDate <= :dateTo';
			$params['dateTo'] = new \DateTime($this->_input['dateTo'] . ' 23:59:59');
		}
		if (isset($this->_input['group'])
				&& !empty($this->_input['group'])
				&& 'null' != $this->_input['group'])
		{
			$this->_queries['Group'] = $this->em
				->find('\Company\Entity\Group', $this->_input['group'])
				->name;
			$where[] = 'sellerGroup.id = :group';
			$params['group'] = $this->_input['group'];
		}
		if (isset($this->_input['groupDivision'])
				&& !empty($this->_input['groupDivision'])
				&& 'null' != $this->_input['groupDivision'])
		{
			error_log($this->_input['groupDivision']);
			$this->_queries['Division'] = $this->em
				->find('\Company\Entity\GroupDivision', $this->_input['groupDivision'])
				->name;
			$where[] = 'sellerDivision.id = :division';
			$params['division'] = $this->_input['groupDivision'];
		}
		if (isset($this->_input['company'])
				&& !empty($this->_input['company'])
				&& 'null' != $this->_input['company'])
		{
			$this->_queries['Dealership'] = $this->em
				->find('\Company\Entity\Company', $this->_input['company'])
				->name;
			$where[] = 'sellerCompany.id = :company';
			$params['company'] = $this->_input['company'];
		}

		#-> Finalize query.
		$query = str_replace(array(
				'[SELECTION]', '[WHERE]'
			), array(
				implode(', ', $selection) . ' ',
				!empty($where)
					? 'WHERE ' . implode(' AND ', $where) . ' '
					: ' '
			), $query);

		#-> Collect data.
		$query = $this->em->createQuery($query);
		!empty($params)
			&& $query->setParameters($params);

		$this->_data = $query->getScalarResult();

		$this->em->clear();
//		foreach ($this->_data as $id => $row)
//		{
//			$this->_data[$id]['bidPerc'] = 0 < $row['stock_tradePrice']
//				? round((100 / $row['stock_tradePrice']) * $row['auction_currentBidPrice'], 1) . ' %'
//				: '0.0 %';
//		}
	}

}
