<?php
namespace Report\Report;



/**
 * Tuhpi Log Report.
 * @author andre.fourie
 */
class TuhpiLog extends \Utility\Service\Report
{



	/**
	 * @var string
	 */
	protected $_title  = 'HPI Log Report';
	/**
	 * @var string
	 */
	protected $_subject = 'HPI';
	/**
	 * @var string
	 */
	protected $_description = 'Historical report for HPI report requests.';
	/**
	 * @var array
	 */
	protected $_notes = array(
			'Confidential information, generated using Bid 4 Cars, for more information visit bid4cars.com.na'
	);
	/**
	 * @var array
	 */
	protected $_headers = array(
			'Group',
			'Division',
			'Dealership',
			'User Name',
			'User Surname',
			'VIN No.',
			'Engine No.',
			'Registration No.',
			'Vehicle Make',
			'Model',
			'Type',
			'MM Code',
			'Date requested',
			'Transaction Number',
			'Status'
	);
	/**
	 * @var array
	 */
	protected $_fields = array(
			'companyGroup_name',
			'companyDivision_name',
			'company_name',
			'profile_firstName',
			'profile_familyName',
			'stock_vinNumber',
			'stock_engineNumber',
			'stock_registrationNumber',
			'make_name',
			'model_name',
			'type_name',
			'type_mmCode',
			'log_created',
			'tuhpi_transactionNumber',
			'log_status'
	);
	/**
	 * @var array
	 */
	protected $_totals = array();
	/**
	 * @var array
	 */
	protected $_currencyFields = array();



	/**
	 * Build the dataset.
	 */
	public function build()
	{
		#-> Prepare parameters.
		$selection = array(
				'log', 'tuhpi', 'stock', 'type', 'model', 'make',
				'company', 'companyGroup', 'companyDivision', 'profile'
		);
		$join      = array();
		$leftJion  = array();
		$where     = array();
		$params    = array();
		$query = 'SELECT [SELECTION] '
				. 'FROM \Stock\Entity\TuhpiLog log '
				. ' JOIN log.stock stock '
				. ' JOIN log.company company '
				. ' JOIN log.createdBy profile '
				. ' LEFT JOIN log.tuhpi tuhpi '
				. ' LEFT JOIN stock.type type '
				. ' LEFT JOIN type.model model '
				. ' LEFT JOIN model.make make '
				. ' LEFT JOIN company.group companyGroup '
				. ' LEFT JOIN company.groupDivision companyDivision '
				. '[WHERE] '
				. 'ORDER BY log.created ASC';

        $where[] = ' (tuhpi.hpi_type is null OR tuhpi.hpi_type = :hpiType) ';
        $params['hpiType'] = 'HPI';

		#-> Construct details.
		$authData = \Utility\Registry::getAuthData();
		if (isset($this->_input['dateFrom']) && !empty($this->_input['dateFrom']))
		{
			$this->_queries['Date Range From'] = $this->_input['dateFrom'];
			$where[] = 'log.created >= :dateFrom';
			$params['dateFrom'] = new \DateTime($this->_input['dateFrom']);
		}
		if (isset($this->_input['dateTo']) && !empty($this->_input['dateTo']))
		{
			$this->_queries['Date Range Until'] = $this->_input['dateTo'];
			$where[] = 'log.created <= :dateTo';
			$params['dateTo'] = new \DateTime($this->_input['dateTo'] . ' 23:59:59');
		}
		if (isset($this->_input['group'])
				&& !empty($this->_input['group'])
				&& 'null' != $this->_input['group'])
		{
			$this->_queries['Group'] = $this->em
				->find('\Company\Entity\Group', $this->_input['group'])
				->name;
			$where[] = 'companyGroup.id = :group';
			$params['group'] = $this->_input['group'];
		}
		if (isset($this->_input['groupDivision'])
				&& !empty($this->_input['groupDivision'])
				&& 'null' != $this->_input['groupDivision'])
		{
//			error_log($this->_input['groupDivision']);
			$this->_queries['Division'] = $this->em
				->find('\Company\Entity\GroupDivision', $this->_input['groupDivision'])
				->name;
			$where[] = 'companyDivision.id = :division';
			$params['division'] = $this->_input['groupDivision'];
		}
		if (isset($this->_input['company'])
				&& !empty($this->_input['company'])
				&& 'null' != $this->_input['company'])
		{
			$this->_queries['Dealership'] = $this->em
				->find('\Company\Entity\Company', $this->_input['company'])
				->name;
			$where[] = 'company.id = :company';
			$params['company'] = $this->_input['company'];
		}

		#-> Finalize query.
		$query = str_replace(array(
				'[SELECTION]', '[WHERE]'
			), array(
				implode(', ', $selection) . ' ',
				!empty($where)
					? 'WHERE ' . implode(' AND ', $where) . ' '
					: ' '
			), $query);

		#-> Collect data.
		$query = $this->em->createQuery($query);
		!empty($params)
			&& $query->setParameters($params);

//        \Utility\Debug::errorLog('$query->getSQL()', $query->getSQL());
//        \Utility\Debug::errorLog('$query->getParameters()', $query->getParameters());


//        \Utility\Debug::errorLog('$query->getScalarResult()', $query->getScalarResult());

		$this->_data = $query->getScalarResult();
		$this->em->clear();
	}

}
