<?php
namespace Stock\Entity;

use Doctrine\ORM\Mapping as ORM;


/**
 * A stock item.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="vehicle_pricing_history")
 */
class PricingHistory
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;
	/**
	 * Post insert action must be called after new entity is flushed to database.
	 */
	const HAVE_POST_INSERT = true;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="datetime", nullable=true, name="date");)
	 */
	protected $date;
	
	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="trade", options={"unsigned"=true});
	 */
	protected $trade = 0.0;

    /**
     * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="list_price", options={"unsigned"=true});
     */
    protected $listPrice = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="total_recon", options={"unsigned"=true});
	 */
	protected $totalRecon = 0.0;
	
	/**
	 * @ORM\Column(type="integer", nullable=false, name="km", options={"unsigned"=true})
	 */
	protected $km = 0;
	
	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="priceg_offer", options={"unsigned"=true});
	 */
	protected $pricegOffer = 0.0;
	
	
	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="auction_winbid", options={"unsigned"=true});
	 */
	protected $auctionWinningBid = 0.0;
	
	
	/**
	 * @ORM\Column(type="integer", nullable=false, name="stock_id");
	 */
	protected $stockId = 0;
	
	
	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array(), array $intersect = array())
	{
		
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['date']))
			&& $data['date'] = $this->date;
		($includeAll || isset($intersect['trade']))
			&& $data['trade'] = $this->trade;
        ($includeAll || isset($intersect['listPrice']))
            && $data['listPrice'] = $this->listPrice;
		($includeAll || isset($intersect['totalRecon']))
			&& $data['totalRecon'] = $this->totalRecon;
		($includeAll || isset($intersect['km']))
			&& $data['km'] = $this->km;
		($includeAll || isset($intersect['pricegOffer']))
			&& $data['pricegOffer'] = $this->pricegOffer;
		($includeAll || isset($intersect['auctionWinningBid']))
			&& $data['auctionWinningBid'] = $this->auctionWinningBid;
		($includeAll || isset($intersect['stockId']))
			&& $data['stockId'] = $this->stockId;
		return $data;
		
	}

	/**
	 * Convert the object to an array for synchronization.
	 * @return array
	 */
	public function toSynchArray()
	{
		return array(
				'id'   => $this->id,
				'date' => $this->date,
				'trade' => $this->trade,
                'listPrice' => $this->listPrice,
				'totalRecon' => $this->totalRecon,
				'km' => $this->km,
				'pricegOffer' => $this->pricegOffer,
				'auctionWinningBid' => $this->auctionWinningBid,
				'stockId' => $this->stockId
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['date'])
			&& $this->date  = $data['date'];
		isset($data['trade'])
			&& $this->trade  = $data['trade'];
        isset($data['listPrice'])
            && $this->listPrice  = $data['listPrice'];
		isset($data['totalRecon'])
			&& $this->totalRecon  = $data['totalRecon'];
		isset($data['km'])
			&& $this->km  = $data['km'];
		isset($data['pricegOffer'])
			&& $this->pricegOffer  = $data['pricegOffer'];
		isset($data['auctionWinningBid'])
			&& $this->auctionWinningBid  = $data['auctionWinningBid'];
		isset($data['stockId'])
			&& $this->stockId  = $data['stockId'];
	}

}
