<?php
namespace Stock\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\Table(name="stock_damages")
 */
class StockDamage
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Build';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Build';


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Stock", inversedBy="damages")
	 * @ORM\JoinColumn(name="stock_id", referencedColumnName="id")
	 **/
	protected $stock;

	/**
	 * @ORM\ManyToOne(targetEntity="Damage")
	 * @ORM\JoinColumn(name="vehicle_damage_id", referencedColumnName="id")
	 **/
	protected $damage;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, options={"unsigned"=true});
	 */
	protected $amount = 0.0;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(), $showIdentifiers = false
			)
	{
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['damage']))
			&& $data['damage'] = $showIdentifiers
									? $this->damage->id
									: $this->damage->toArray();
		($includeAll || isset($intersect['amount']))
			&& $data['amount'] = $this->amount;
		return $data;
	}

	/**
	 * Convert the object to an array for synchronization.
	 * @return array
	 */
	public function toSynchArray()
	{
		return array(
				'id'   => $this->id,
				'name' => $this->name
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['stock'])
			&& $this->stock  = $data['stock'];
		isset($data['damage'])
			&& $this->damage  = $data['damage'];
		isset($data['amount'])
			&& $this->amount  = $data['amount'];
	}

}
