<?php
namespace Stock\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * @ORM\Entity
 * @ORM\Table(name="stock_images")
 */
class StockImages
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;


	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="Stock", inversedBy="accessories")
	 * @ORM\JoinColumn(name="stock_id", referencedColumnName="id")
	 **/
	protected $stock;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="main_lib_photo_id")
	 **/
	protected $mainImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="main_url");
	 */
	protected $mainImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="front_lib_photo_id")
	 **/
	protected $frontImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="front_url");
	 */
	protected $frontImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="right_lib_photo_id")
	 **/
	protected $rightImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="right_url");
	 */
	protected $rightImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="left_lib_photo_id")
	 **/
	protected $leftImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="left_url");
	 */
	protected $leftImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="back_lib_photo_id")
	 **/
	protected $backImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="back_url");
	 */
	protected $backImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="interior_lib_photo_id")
	 **/
	protected $interiorImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="interior_url");
	 */
	protected $interiorImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="engine_lib_photo_id")
	 **/
	protected $engineImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="engine_url");
	 */
	protected $engineImageUrl;

	/**
	 * @ORM\ManyToOne(targetEntity="\Utility\Entity\Image")
	 * @ORM\JoinColumn(nullable=true, name="natis_lib_photo_id")
	 **/
	protected $natisImage;
	/**
	 * @ORM\Column(type="string", nullable=true, length=255, name="natis_url");
	 */
	protected $natisImageUrl;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(), $showIdentifiers = false
			)
	{
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['accessory']))
			&& $data['accessory'] = $showIdentifiers
									? $this->accessory->id
									: $this->accessory->toArray();
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['stock'])
			&& $this->stock  = $data['stock'];
		isset($data['mainImage'])
			&& $this->mainImage  = $data['mainImage'];
		isset($data['mainImageUrl'])
			&& $this->mainImageUrl  = $data['mainImageUrl'];
		isset($data['frontImage'])
			&& $this->frontImage  = $data['frontImage'];
		isset($data['frontImageUrl'])
			&& $this->frontImageUrl  = $data['frontImageUrl'];
		isset($data['rightImage'])
			&& $this->rightImage  = $data['rightImage'];
		isset($data['rightImageUrl'])
			&& $this->rightImageUrl  = $data['rightImageUrl'];
		isset($data['leftImage'])
			&& $this->leftImage  = $data['leftImage'];
		isset($data['leftImageUrl'])
			&& $this->leftImageUrl  = $data['leftImageUrl'];
		isset($data['backImage'])
			&& $this->backImage  = $data['backImage'];
		isset($data['backImageUrl'])
			&& $this->backImageUrl  = $data['backImageUrl'];
		isset($data['interiorImage'])
			&& $this->interiorImage  = $data['interiorImage'];
		isset($data['interiorImageUrl'])
			&& $this->interiorImageUrl  = $data['interiorImageUrl'];
		isset($data['engineImage'])
			&& $this->engineImage  = $data['engineImage'];
		isset($data['engineImageUrl'])
			&& $this->engineImageUrl  = $data['engineImageUrl'];
		isset($data['natisImage'])
			&& $this->natisImage  = $data['natisImage'];
		isset($data['natisImageUrl'])
			&& $this->natisImageUrl  = $data['natisImageUrl'];
	}

}
