<?php
namespace Stock\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A Trans Union HPI vs stock item.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="tuhpi")
 */
class Tuhpi
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;
	/**
	 * Post insert action must be called after new entity is flushed to database.
	 */
	const HAVE_POST_INSERT = true;



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=false, name="company_id")
	 **/
	protected $company;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="created_by_profile_id")
	 **/
	protected $createdBy;

	/**
	 * @ORM\ManyToOne(targetEntity="\Valuation\Entity\Valuation", cascade={"all"})
	 * @ORM\JoinColumn(nullable=true, name="stock_valuation_id")
	 **/
	protected $valuation;

	/**
	 * @ORM\ManyToOne(targetEntity="\PriceGuide\Entity\PriceGuide")
	 * @ORM\JoinColumn(nullable=true, name="price_guide_id")
	 **/
	protected $priceGuide;

	/**
	 * @ORM\ManyToOne(targetEntity="\Auction\Entity\Auction")
	 * @ORM\JoinColumn(nullable=true, name="auction_id")
	 **/
	protected $auction;


	/**
	 * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock")
	 * @ORM\JoinColumn(nullable=false, name="stock_id")
	 **/
	protected $stock;


	/* ------------------------------------ Finance Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=15, nullable=true, name="transaction_number")
	 */
	protected $transactionNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_financeProvider")
	 */
	protected $f_financeProvider;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_financeBranch")
	 */
	protected $f_financeBranch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_agreementOrAccountNumber")
	 */
	protected $f_agreementOrAccountNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_agreementType")
	 */
	protected $f_agreementType;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_telephoneNumber")
	 */
	protected $f_telephoneNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="f_resultCode")
	 */
	protected $f_resultCode;


	/* ------------------------------------ Finance History ------------------------------------ */

	/**
	 * @ORM\Column(type="text", name="fh_data")
	 */
	protected $fh_data;


	/* ------------------------------------ Accident History ------------------------------------ */


	/**
	 * @ORM\Column(type="text", name="ah_data")
	 */
	protected $ah_data;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_assessmentDate")
	 */
	protected $ah_assessmentDate;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_vehicleDescription")
	 */
	protected $ah_vehicleDescription;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_vinMatch")
	 */
	protected $ah_vinMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_regMatch")
	 */
	protected $ah_regMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_estimatedRepairCost")
	 */
	protected $ah_estimatedRepairCost;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_assessmentNumber")
	 */
	protected $ah_assessmentNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_engineMatch")
	 */
	protected $ah_engineMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_estimatePercentageOfRetailValue")
	 */
	protected $ah_estimatePercentageOfRetailValue;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ah_resultCode")
	 */
	protected $ah_resultCode;


	/* ------------------------------------ Stolen Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_policeStation")
	 */
	protected $si_policeStation;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_caseNumber")
	 */
	protected $si_caseNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_dateStolen")
	 */
	protected $si_dateStolen;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_vinMatch")
	 */
	protected $si_vinMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_engineMatch")
	 */
	protected $si_engineMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_registrationMatch")
	 */
	protected $si_registrationMatch;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="si_resultCode")
	 */
	protected $si_resultCode;


	/* ------------------------------------ Alert Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ai_company")
	 */
	protected $ai_company;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ai_referenceNumber")
	 */
	protected $ai_referenceNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ai_alertReason")
	 */
	protected $ai_alertReason;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ai_telephoneNumber")
	 */
	protected $ai_telephoneNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ai_resultCode")
	 */
	protected $ai_resultCode;


	/* ------------------------------------ IVID History Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ivid_responseCode")
	 */
	protected $ivid_responseCode;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ivid_responseDescription")
	 */
	protected $ivid_responseDescription;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ivid_referenceNumber")
	 */
	protected $ivid_referenceNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ivid_contactNumber")
	 */
	protected $ivid_contactNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="ivid_resultCode")
	 */
	protected $ivid_resultCode;


	/* ------------------------------------ Microdot Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="mi_company")
	 */
	protected $mi_company;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="mi_referenceNumber")
	 */
	protected $mi_referenceNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="mi_contactNumber")
	 */
	protected $mi_contactNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="mi_dateApplied")
	 */
	protected $mi_dateApplied;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="mi_resultCode")
	 */
	protected $mi_resultCode;


	/* ------------------------------------ VESA Information ------------------------------------ */


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_CertificateNumber")
	 */
	protected $vesa_CertificateNumber;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_dateFitted")
	 */
	protected $vesa_dateFitted;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_brandName")
	 */
	protected $vesa_brandName;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_brandModel")
	 */
	protected $vesa_brandModel;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_deviceDescriptionType")
	 */
	protected $vesa_deviceDescriptionType;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vesa_resultCode")
	 */
	protected $vesa_resultCode;


	/* ------------------------------------ Registration History ------------------------------------ */

	/**
	 * @ORM\Column(type="text", name="rh_data")
	 */
	protected $rh_data;



	/* ------------------------------------ Enquiry History ------------------------------------ */


	/**
	 * @ORM\Column(type="text", name="eh_data")
	 */
	protected $eh_data;



	/* ------------------------------------ Mileage History ------------------------------------ */


	/**
	 * @ORM\Column(type="text", name="mh_data")
	 */
	protected $mh_data;

	/* ------------------------------------ VehicleOptionCodes ------------------------------------ */


	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_month");
	 */
	protected $vc_month = 0;


	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_year");
	 */
	protected $vc_year = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_newOptionCode");
	 */
	protected $vc_newOptionCode = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_usedOptionCode");
	 */
	protected $vc_usedOptionCode = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_maximumRange");
	 */
	protected $vc_maximumRange = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_minimunRange");
	 */
	protected $vc_minimunRange = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_newPrice");
	 */
	protected $vc_newPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_retailPrice");
	 */
	protected $vc_retailPrice = 0;


	/**
	 * @ORM\Column(type="integer", nullable=false, name="vc_tradePrice");
	 */
	protected $vc_tradePrice = 0;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vc_resultCode")
	 */
	protected $vc_resultCode;



	/* ------------------------------------ VehicleOptionCodes ------------------------------------ */
	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_hpiNumber");
	 */
	protected $vci_HPINumber;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchColour");
	 */
	protected $vci_matchColour;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchEngineNumber");
	 */
	protected $vci_matchEngineNumber;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchManufacturer");
	 */
	protected $vci_matchManufacturer;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchModel");
	 */
	protected $vci_matchModel;

	/**
	 * @ORM\Column(type="string", length=250, nullable=true, name="vci_matchString");
	 */
	protected $vci_matchString;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchVehicleRegistration");
	 */
	protected $vci_matchVehicleRegistration;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchVinorChassis");
	 */
	protected $vci_matchVinorChassis;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="vci_matchYear");
	 */
	protected $vci_matchYear;



	/* ------------------------------------ VehicleValueInfo ------------------------------------ */



	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_guideMonth");
	 */
	protected $vvi_guideMonth = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_guideYear");
	 */
	protected $vvi_guideYear = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_newListPrice");
	 */
	protected $vvi_newListPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_retailPrice");
	 */
	protected $vvi_retailPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_tradePrice");
	 */
	protected $vvi_tradePrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_costPrice");
	 */
	protected $vvi_costPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedRetailPrice_mileageAndCondition");
	 */
	protected $vvi_adjustedRetailPrice_mileageAndCondition = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_AdjustedTradePrice_MileageAndCondition");
	 */
	protected $vvi_AdjustedTradePrice_MileageAndCondition = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedRetailPrice");
	 */
	protected $vvi_adjustedRetailPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedTradePrice");
	 */
	protected $vvi_adjustedTradePrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_estimatedRetailPrice");
	 */
	protected $vvi_estimatedRetailPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_estimatedTradePrice");
	 */
	protected $vvi_estimatedTradePrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_estimatedCostPrice");
	 */
	protected $vvi_estimatedCostPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedRetailPrice_mileageAndCondition");
	 */
	protected $vvi_adjustedEstimatedRetailPrice_mileageAndCondition = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedTradePrice_mileageAndCondition");
	 */
	protected $vvi_adjustedEstimatedTradePrice_mileageAndCondition = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedcostprice_mileageandcondition");
	 */
	protected $vvi_adjustedEstimatedcostprice_mileageandcondition = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedRetailPrice");
	 */
	protected $vvi_adjustedEstimatedRetailPrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedTradePrice");
	 */
	protected $vvi_adjustedEstimatedTradePrice = 0;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="vvi_adjustedEstimatedCostPrice");
	 */
	protected $vvi_adjustedEstimatedCostPrice = 0;


	/**
	 * @ORM\Column(type="string", length=255, nullable=true, name="vvi_resultCode")
	 */
	protected $vvi_resultCode;


    /**
     * @ORM\Column(type="string", length=255, nullable=true, name="hpi_type")
     */
    protected $hpi_type;

	/* ------------------------------------ Error codes ------------------------------------ */


	/**
	 * @ORM\Column(type="integer", nullable=true, name="error_code");
	 */
	protected $error_code = 0;



	/* ------------------------------------ Tracking ------------------------------------ */


	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $reset;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="previous_status");
	 */
	protected $previousState;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="status");
	 */
	protected $jobState;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;



	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->accessories = new \Doctrine\Common\Collections\ArrayCollection();
		$this->damages = new \Doctrine\Common\Collections\ArrayCollection();
	}


	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		if (\Utility\Registry::isAuthenticated())
		{
			$this->company   = \Utility\Registry::resolveCompanyContext($this->company);
			$this->createdBy = \Utility\Registry::resolveProfileContext($this->createdBy);
		}
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
		/* if ('' == $this->jobState
			&& is_null($this->loadedOnPriceGuide))
		{
			$this->loadedOnPriceGuide = new \DateTime("now");
		} */
	}



	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;

		($includeAll || isset($intersect['company']))
			&& $data['company'] = (in_array('company', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->company)
				? (!$showIdentifiers || in_array('company', $expand) ? $this->company->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->company->id)
				: null;
		($includeAll || isset($intersect['createdBy']))
			&& $data['createdBy'] = (in_array('createdBy', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->createdBy)
				? (!$showIdentifiers || in_array('createdBy', $expand) ? $this->createdBy->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->createdBy->id)
				: null;
		($includeAll || isset($intersect['valuation']))
			&& $data['valuation'] = (in_array('valuation_', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->valuation)
				? (!$showIdentifiers || in_array('valuation_', $expand) ? $this->valuation->toArray(
						/* $expand, $intersect, $showIdentifiers, ($expandAll - 1) */
						) : $this->valuation->id)
				: null;

		($includeAll || isset($intersect['priceGuide']))
			&& $data['priceGuide'] = (in_array('priceGuide', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->priceGuide)
				? (!$showIdentifiers || in_array('priceGuide', $expand) ? $this->priceGuide->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->priceGuide->id)
				: null;

		($includeAll || isset($intersect['auction']))
			&& $data['auction'] = (in_array('auction', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->auction)
			? (!$showIdentifiers || in_array('auction', $expand) ? $this->auction->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
				) : $this->auction->id)
			: null;

		($includeAll || isset($intersect['stock']))
			&& $data['stock'] = (in_array('stock', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->stock)
			? (!$showIdentifiers || in_array('stock', $expand) ? $this->stock->toArray(
				$expand, $intersect, $showIdentifiers, ($expandAll - 1)
				) : $this->stock->id)
				: null;


		if (2 == $showIdentifiers)
		{
			$data['mmCode']   = $this->type->mmCode;
			$data['category'] = $this->type->category->id;
			$data['make']     = $this->type->model->make->id;
			$data['model']    = $this->type->model->id;
		}




	/* 	($includeAll || isset($intersect['referenceNumber']))
		&& $data['referenceNumber'] = $this->referenceNumber; */
		($includeAll || isset($intersect['transactionNumber']))
		&& $data['transactionNumber'] = $this->transactionNumber;

		/* ------------------------------------ Finance Information ------------------------------------ */

		($includeAll || isset($intersect['f_financeProvider']))
		&& $data['f_financeProvider'] = $this->f_financeProvider;


		($includeAll || isset($intersect['f_financeBranch']))
		&& $data['f_financeBranch'] = $this->f_financeBranch;

		($includeAll || isset($intersect['f_agreementOrAccountNumber']))
		&& $data['f_agreementOrAccountNumber'] = $this->f_agreementOrAccountNumber;

		($includeAll || isset($intersect['f_agreementType']))
		&& $data['f_agreementType'] = $this->f_agreementType;

		($includeAll || isset($intersect['f_telephoneNumber']))
		&& $data['f_telephoneNumber'] = $this->f_telephoneNumber;

		($includeAll || isset($intersect['f_resultCode']))
		&& $data['f_resultCode'] = $this->f_resultCode;


		/* ------------------------------------ Finance History ------------------------------------ */


		($includeAll || isset($intersect['fh_data']))
		&& $data['fh_data'] = unserialize($this->fh_data);


		/* ------------------------------------ Accident History ------------------------------------ */

		($includeAll || isset($intersect['ah_data']))
		&& $data['ah_data'] = unserialize($this->ah_data);



		/* ------------------------------------ Stolen Information ------------------------------------ */

		($includeAll || isset($intersect['si_policeStation']))
		&& $data['si_policeStation'] = $this->si_policeStation;

		($includeAll || isset($intersect['si_caseNumber']))
		&& $data['si_caseNumber'] = $this->si_caseNumber;

		($includeAll || isset($intersect['si_dateStolen']))
		&& $data['si_dateStolen'] = $this->si_dateStolen;

		($includeAll || isset($intersect['si_vinMatch']))
		&& $data['si_vinMatch'] = $this->si_vinMatch;

		($includeAll || isset($intersect['si_engineMatch']))
		&& $data['si_engineMatch'] = $this->si_engineMatch;

		($includeAll || isset($intersect['si_registrationMatch']))
		&& $data['si_registrationMatch'] = $this->si_registrationMatch;

		($includeAll || isset($intersect['si_resultCode']))
		&& $data['si_resultCode'] = $this->si_resultCode;


		/* ------------------------------------ Alert Information ------------------------------------ */

		($includeAll || isset($intersect['ai_company']))
		&& $data['ai_company'] = $this->ai_company;

		($includeAll || isset($intersect['ai_referenceNumber']))
		&& $data['ai_referenceNumber'] = $this->ai_referenceNumber;

		($includeAll || isset($intersect['ai_alertReason']))
		&& $data['ai_alertReason'] = $this->ai_alertReason;

		($includeAll || isset($intersect['ai_telephoneNumber']))
		&& $data['ai_telephoneNumber'] = $this->ai_telephoneNumber;

		($includeAll || isset($intersect['ai_resultCode']))
		&& $data['ai_resultCode'] = $this->ai_resultCode;


		/* ------------------------------------ IVID History Information ------------------------------------ */

		($includeAll || isset($intersect['ivid_responseCode']))
		&& $data['ivid_responseCode'] = $this->ivid_responseCode;

		($includeAll || isset($intersect['ivid_responseDescription']))
		&& $data['ivid_responseDescription'] = $this->ivid_responseDescription;

		($includeAll || isset($intersect['ivid_referenceNumber']))
		&& $data['ivid_referenceNumber'] = $this->ivid_referenceNumber;

		($includeAll || isset($intersect['ivid_contactNumber']))
		&& $data['ivid_contactNumber'] = $this->ivid_contactNumber;

		($includeAll || isset($intersect['ivid_resultCode']))
		&& $data['ivid_resultCode'] = $this->ivid_resultCode;


		/* ------------------------------------ Microdot Information ------------------------------------ */

		($includeAll || isset($intersect['mi_company']))
		&& $data['mi_company'] = $this->mi_company;

		($includeAll || isset($intersect['mi_referenceNumber']))
		&& $data['mi_referenceNumber'] = $this->mi_referenceNumber;

		($includeAll || isset($intersect['mi_contactNumber']))
		&& $data['mi_contactNumber'] = $this->mi_contactNumber;

		($includeAll || isset($intersect['mi_dateApplied']))
		&& $data['mi_dateApplied'] = $this->mi_dateApplied;

		($includeAll || isset($intersect['mi_resultCode']))
		&& $data['mi_resultCode'] = $this->mi_resultCode;



		/* ------------------------------------ VESA Information ------------------------------------ */

		($includeAll || isset($intersect['vesa_CertificateNumber']))
		&& $data['vesa_CertificateNumber'] = $this->vesa_CertificateNumber;

		($includeAll || isset($intersect['vesa_dateFitted']))
		&& $data['vesa_dateFitted'] = $this->vesa_dateFitted;

		($includeAll || isset($intersect['vesa_brandName']))
		&& $data['vesa_brandName'] = $this->vesa_brandName;

		($includeAll || isset($intersect['vesa_brandModel']))
		&& $data['vesa_brandModel'] = $this->vesa_brandModel;

		($includeAll || isset($intersect['vesa_deviceDescriptionType']))
		&& $data['vesa_deviceDescriptionType'] = $this->vesa_deviceDescriptionType;

		($includeAll || isset($intersect['vesa_resultCode']))
		&& $data['vesa_resultCode'] = $this->vesa_resultCode;



		/* ------------------------------------ Registration History ------------------------------------ */

		($includeAll || isset($intersect['rh_data']))
		&& $data['rh_data'] = unserialize($this->rh_data);



		/* ------------------------------------ Enquiry History ------------------------------------ */

		($includeAll || isset($intersect['eh_data']))
		&& $data['eh_data'] = unserialize($this->eh_data);



		/* ------------------------------------ Mileage History ------------------------------------ */

		($includeAll || isset($intersect['mh_data']))
		&& $data['mh_data'] = unserialize($this->mh_data);


		/* ------------------------------------ VehicleOptionCodes ------------------------------------ */

		($includeAll || isset($intersect['vc_month']))
		&& $data['vc_month'] = $this->vc_month;

		($includeAll || isset($intersect['vc_year']))
		&& $data['vc_year'] = $this->vc_year;

		($includeAll || isset($intersect['vc_newOptionCode']))
		&& $data['vc_newOptionCode'] = $this->vc_newOptionCode;

		($includeAll || isset($intersect['vc_usedOptionCode']))
		&& $data['vc_usedOptionCode'] = $this->vc_usedOptionCode;

		($includeAll || isset($intersect['vc_maximumRange']))
		&& $data['vc_maximumRange'] = $this->vc_maximumRange;

		($includeAll || isset($intersect['vc_minimunRange']))
		&& $data['vc_minimunRange'] = $this->vc_minimunRange;

		($includeAll || isset($intersect['vc_newPrice']))
		&& $data['vc_newPrice'] = $this->vc_newPrice;

		($includeAll || isset($intersect['vc_retailPrice']))
		&& $data['vc_retailPrice'] = $this->vc_retailPrice;

		($includeAll || isset($intersect['vc_tradePrice']))
		&& $data['vc_tradePrice'] = $this->vc_tradePrice;


		($includeAll || isset($intersect['vc_resultCode']))
		&& $data['vc_resultCode'] = $this->vc_resultCode;



		/* ------------------------------------ VehicleConfirmationInfo ------------------------------------ */

		($includeAll || isset($intersect['vci_HPINumber']))
		&& $data['vci_HPINumber'] = $this->vci_HPINumber;

		($includeAll || isset($intersect['vci_matchColour']))
		&& $data['vci_matchColour'] = $this->vci_matchColour;

		($includeAll || isset($intersect['vci_matchEngineNumber']))
		&& $data['vci_matchEngineNumber'] = $this->vci_matchEngineNumber;

		($includeAll || isset($intersect['vci_matchManufacturer']))
		&& $data['vci_matchManufacturer'] = $this->vci_matchManufacturer;

		($includeAll || isset($intersect['vci_matchModel']))
		&& $data['vci_matchModel'] = $this->vci_matchModel;

		($includeAll || isset($intersect['vci_matchString']))
		&& $data['vci_matchString'] = $this->vci_matchString;

		($includeAll || isset($intersect['vci_matchVehicleRegistration']))
		&& $data['vci_matchVehicleRegistration'] = $this->vci_matchVehicleRegistration;

		($includeAll || isset($intersect['vci_matchVinorChassis']))
		&& $data['vci_matchVinorChassis'] = $this->vci_matchVinorChassis;

		($includeAll || isset($intersect['vci_matchYear']))
		&& $data['vci_matchYear'] = $this->vci_matchYear;

		/* ------------------------------------ VehicleValueInfo ------------------------------------ */

		($includeAll || isset($intersect['vvi_guideMonth']))
		&& $data['vvi_guideMonth'] = $this->vvi_guideMonth;

		($includeAll || isset($intersect['vvi_guideYear']))
		&& $data['vvi_guideYear'] = $this->vvi_guideYear;

		($includeAll || isset($intersect['vvi_newListPrice']))
		&& $data['vvi_newListPrice'] = $this->vvi_newListPrice;

		($includeAll || isset($intersect['vvi_retailPrice']))
		&& $data['vvi_retailPrice'] = $this->vvi_retailPrice;

		($includeAll || isset($intersect['vvi_tradePrice']))
		&& $data['vvi_tradePrice'] = $this->vvi_tradePrice;

		($includeAll || isset($intersect['vvi_costPrice']))
		&& $data['vvi_costPrice'] = $this->vvi_costPrice;

		($includeAll || isset($intersect['vvi_adjustedRetailPrice_mileageAndCondition']))
		&& $data['vvi_adjustedRetailPrice_mileageAndCondition'] = $this->vvi_adjustedRetailPrice_mileageAndCondition;

		($includeAll || isset($intersect['vvi_AdjustedTradePrice_MileageAndCondition']))
		&& $data['vvi_AdjustedTradePrice_MileageAndCondition'] = $this->vvi_AdjustedTradePrice_MileageAndCondition;

		($includeAll || isset($intersect['vvi_adjustedRetailPrice']))
		&& $data['vvi_adjustedRetailPrice'] = $this->vvi_adjustedRetailPrice;

		($includeAll || isset($intersect['vvi_adjustedTradePrice']))
		&& $data['vvi_adjustedTradePrice'] = $this->vvi_adjustedTradePrice;

		($includeAll || isset($intersect['vvi_estimatedRetailPrice']))
		&& $data['vvi_estimatedRetailPrice'] = $this->vvi_estimatedRetailPrice;

		($includeAll || isset($intersect['vvi_estimatedTradePrice']))
		&& $data['vvi_estimatedTradePrice'] = $this->vvi_estimatedTradePrice;

		($includeAll || isset($intersect['vvi_estimatedCostPrice']))
		&& $data['vvi_estimatedCostPrice'] = $this->vvi_estimatedCostPrice;

		($includeAll || isset($intersect['vvi_adjustedEstimatedRetailPrice_mileageAndCondition']))
		&& $data['vvi_adjustedEstimatedRetailPrice_mileageAndCondition'] = $this->vvi_adjustedEstimatedRetailPrice_mileageAndCondition;

		($includeAll || isset($intersect['vvi_adjustedEstimatedTradePrice_mileageAndCondition']))
		&& $data['vvi_adjustedEstimatedTradePrice_mileageAndCondition'] = $this->vvi_adjustedEstimatedTradePrice_mileageAndCondition;

		($includeAll || isset($intersect['vvi_adjustedEstimatedcostprice_mileageandcondition']))
		&& $data['vvi_adjustedEstimatedcostprice_mileageandcondition'] = $this->vvi_adjustedEstimatedcostprice_mileageandcondition;

		($includeAll || isset($intersect['vvi_adjustedEstimatedRetailPrice']))
		&& $data['vvi_adjustedEstimatedRetailPrice'] = $this->vvi_adjustedEstimatedRetailPrice;

		($includeAll || isset($intersect['vvi_adjustedEstimatedTradePrice']))
		&& $data['vvi_adjustedEstimatedTradePrice'] = $this->vvi_adjustedEstimatedTradePrice;

		($includeAll || isset($intersect['vvi_adjustedEstimatedCostPrice']))
		&& $data['vvi_adjustedEstimatedCostPrice'] = $this->vvi_adjustedEstimatedCostPrice;

		($includeAll || isset($intersect['vvi_resultCode']))
		&& $data['vvi_resultCode'] = $this->vvi_resultCode;

        ($includeAll || isset($intersect['hpi_type']))
        && $data['hpi_type'] = $this->hpi_type;

		/* ------------------------------------ Error codes ------------------------------------ */

		($includeAll || isset($intersect['error_code']))
		&& $data['error_code'] = $this->error_code;

		($includeAll || isset($intersect['jobState']))
			&& $data['jobState'] = $this->jobState;

		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['reset']))
			&& $data['reset'] = !is_null($this->reset)
				? $this->reset->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Internal utility to change accessories collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	public function accessoriesToArray($showIdentifiers = false)
	{
		$data = array();
		$iterator = $this->accessories->getIterator();
		if (2 == $showIdentifiers)
		{
			foreach ($iterator as $accessory)
			{
				$data[] = $accessory->accessory->id;
			}
		}
		else
		{
			foreach ($iterator as $accessory)
			{
				$data[] = $showIdentifiers
					? array('id' => $accessory->accessory->id)
					: $accessory->toArray();
			}
		}
		return $data;
	}

	/**
	 * Internal utility to change damages collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	public function damagesToArray($showIdentifiers = false)
	{
		$data = array();
		$iterator = $this->damages->getIterator();
		foreach ($iterator as $damage)
		{
			$data[] = $damage->toArray(array(), array(), $showIdentifiers);
		}
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['company'])
			&& $this->company = $data['company'];

		isset($data['createdBy'])
			&& $this->createdBy = $data['createdBy'];

		isset($data['valuation'])
		&& $this->valuation = $data['valuation'];

		isset($data['priceGuide'])
		&& $this->priceGuide = $data['priceGuide'];

		isset($data['auction'])
		&& $this->auction = $data['auction'];

		isset($data['stock'])
		&& $this->stock = $data['stock'];

		isset($data['transactionNumber'])
		&& $this->transactionNumber = $data['transactionNumber'];


		/* ------------------------------------ Finance Information ------------------------------------ */

		isset($data['f_financeProvider'])
		&& $this->f_financeProvider = $data['f_financeProvider'];

		isset($data['f_financeBranch'])
		&& $this->f_financeBranch = $data['f_financeBranch'];

		isset($data['f_agreementOrAccountNumber'])
		&& $this->f_agreementOrAccountNumber = $data['f_agreementOrAccountNumber'];

		isset($data['f_agreementType'])
		&& $this->f_agreementType = $data['f_agreementType'];

		isset($data['f_telephoneNumber'])
		&& $this->f_telephoneNumber = $data['f_telephoneNumber'];

		isset($data['f_resultCode'])
		&& $this->f_resultCode = $data['f_resultCode'];




		/* ------------------------------------ Finance History ------------------------------------ */


		isset($data['fh_data'])
		&& $this->fh_data = is_array($data['fh_data'])
			? serialize($data['fh_data'])
			: serialize(array());



		/* ------------------------------------ Accident History ------------------------------------ */

		isset($data['ah_data'])
		&& $this->ah_data = is_array($data['ah_data'])
			? serialize($data['ah_data'])
			: serialize(array());


		/* ------------------------------------ Stolen Information ------------------------------------ */

		isset($data['si_policeStation'])
		&& $this->si_policeStation = $data['si_policeStation'];

		isset($data['si_caseNumber'])
		&& $this->si_caseNumber = $data['si_caseNumber'];

		isset($data['si_dateStolen'])
		&& $this->si_dateStolen = $data['si_dateStolen'];

		isset($data['si_vinMatch'])
		&& $this->si_vinMatch = $data['si_vinMatch'];

		isset($data['si_engineMatch'])
		&& $this->si_engineMatch = $data['si_engineMatch'];

		isset($data['si_registrationMatch'])
		&& $this->si_registrationMatch = $data['si_registrationMatch'];

		isset($data['si_resultCode'])
		&& $this->si_resultCode = $data['si_resultCode'];


		/* ------------------------------------ Alert Information ------------------------------------ */

		isset($data['ai_company'])
		&& $this->ai_company = $data['ai_company'];

		isset($data['ai_referenceNumber'])
		&& $this->ai_referenceNumber = $data['ai_referenceNumber'];

		isset($data['ai_alertReason'])
		&& $this->ai_alertReason = $data['ai_alertReason'];

		isset($data['ai_telephoneNumber'])
		&& $this->ai_telephoneNumber = $data['ai_telephoneNumber'];

		isset($data['ai_resultCode'])
		&& $this->ai_resultCode = $data['ai_resultCode'];

		/* ------------------------------------ IVID History Information ------------------------------------ */

		isset($data['ivid_responseCode'])
		&& $this->ivid_responseCode = $data['ivid_responseCode'];

		isset($data['ivid_responseDescription'])
		&& $this->ivid_responseDescription = $data['ivid_responseDescription'];

		isset($data['ivid_referenceNumber'])
		&& $this->ivid_referenceNumber = $data['ivid_referenceNumber'];

		isset($data['ivid_contactNumber'])
		&& $this->ivid_contactNumber = $data['ivid_contactNumber'];

		isset($data['ivid_resultCode'])
		&& $this->ivid_resultCode = $data['ivid_resultCode'];

		/* ------------------------------------ Microdot Information ------------------------------------ */

		isset($data['mi_company'])
		&& $this->mi_company = $data['mi_company'];

		isset($data['mi_referenceNumber'])
		&& $this->mi_referenceNumber = $data['mi_referenceNumber'];

		isset($data['mi_contactNumber'])
		&& $this->mi_contactNumber = $data['mi_contactNumber'];

		isset($data['mi_dateApplied'])
		&& $this->mi_dateApplied = $data['mi_dateApplied'];

		isset($data['mi_resultCode'])
		&& $this->mi_resultCode = $data['mi_resultCode'];



		/* ------------------------------------ VESA Information ------------------------------------ */

		isset($data['vesa_CertificateNumber'])
		&& $this->vesa_CertificateNumber = $data['vesa_CertificateNumber'];

		isset($data['vesa_dateFitted'])
		&& $this->vesa_dateFitted = $data['vesa_dateFitted'];

		isset($data['vesa_brandName'])
		&& $this->vesa_brandName = $data['vesa_brandName'];

		isset($data['vesa_brandModel'])
		&& $this->vesa_brandModel = $data['vesa_brandModel'];

		isset($data['vesa_deviceDescriptionType'])
		&& $this->vesa_deviceDescriptionType = $data['vesa_deviceDescriptionType'];

		isset($data['vesa_resultCode'])
		&& $this->vesa_resultCode = $data['vesa_resultCode'];


		/* ------------------------------------ Registration History ------------------------------------ */

		isset($data['rh_data'])
		&& $this->rh_data = is_array($data['rh_data'])
			? serialize($data['rh_data'])
			: serialize(array());


		/* ------------------------------------ Enquiry History ------------------------------------ */

		isset($data['eh_data'])
		&& $this->eh_data = is_array($data['eh_data'])
			? serialize($data['eh_data'])
			: serialize(array());


		/* ------------------------------------ Mileage History ------------------------------------ */

		isset($data['mh_data'])
		&& $this->mh_data = is_array($data['mh_data'])
			? serialize($data['mh_data'])
			: serialize(array());


		/* ------------------------------------ VehicleOptionCodes ------------------------------------ */

		isset($data['vc_month'])
		&& $this->vc_month = $data['vc_month'];

		isset($data['vc_year'])
		&& $this->vc_year = $data['vc_year'];

		isset($data['vc_newOptionCode'])
		&& $this->vc_newOptionCode = $data['vc_newOptionCode'];

		isset($data['vc_usedOptionCode'])
		&& $this->vc_usedOptionCode = $data['vc_usedOptionCode'];

		isset($data['vc_maximumRange'])
		&& $this->vc_maximumRange = $data['vc_maximumRange'];

		isset($data['vc_minimunRange'])
		&& $this->vc_minimunRange = $data['vc_minimunRange'];

		isset($data['vc_newPrice'])
		&& $this->vc_newPrice = $data['vc_newPrice'];

		isset($data['vc_retailPrice'])
		&& $this->vc_retailPrice = $data['vc_retailPrice'];

		isset($data['vc_tradePrice'])
		&& $this->vc_tradePrice = $data['vc_tradePrice'];

		isset($data['vc_resultCode'])
		&& $this->vc_resultCode = $data['vc_resultCode'];


		/* ------------------------------------ VehicleConfirmationInfo ------------------------------------ */

		isset($data['vci_HPINumber'])
		&& $this->vci_HPINumber = $data['vci_HPINumber'];

		isset($data['vci_matchColour'])
		&& $this->vci_matchColour = $data['vci_matchColour'];

		isset($data['vci_matchEngineNumber'])
		&& $this->vci_matchEngineNumber = $data['vci_matchEngineNumber'];

		isset($data['vci_matchManufacturer'])
		&& $this->vci_matchManufacturer = $data['vci_matchManufacturer'];

		isset($data['vci_matchModel'])
		&& $this->vci_matchModel = $data['vci_matchModel'];

		isset($data['vci_matchString'])
		&& $this->vci_matchString = $data['vci_matchString'];

		isset($data['vci_matchVehicleRegistration'])
		&& $this->vci_matchVehicleRegistration = $data['vci_matchVehicleRegistration'];

		isset($data['vci_matchVinorChassis'])
		&& $this->vci_matchVinorChassis = $data['vci_matchVinorChassis'];

		isset($data['vci_matchYear'])
		&& $this->vci_matchYear = $data['vci_matchYear'];


		/* ------------------------------------ VehicleValueInfo ------------------------------------ */

		isset($data['vvi_guideMonth'])
		&& $this->vvi_guideMonth = $data['vvi_guideMonth'];

		isset($data['vvi_guideYear'])
		&& $this->vvi_guideYear = $data['vvi_guideYear'];

		isset($data['vvi_newListPrice'])
		&& $this->vvi_newListPrice = $data['vvi_newListPrice'];

		isset($data['vvi_retailPrice'])
		&& $this->vvi_retailPrice = $data['vvi_retailPrice'];

		isset($data['vvi_tradePrice'])
		&& $this->vvi_tradePrice = $data['vvi_tradePrice'];

		isset($data['vvi_costPrice'])
		&& $this->vvi_costPrice = $data['vvi_costPrice'];

		isset($data['vvi_adjustedRetailPrice_mileageAndCondition'])
		&& $this->vvi_adjustedRetailPrice_mileageAndCondition = $data['vvi_adjustedRetailPrice_mileageAndCondition'];

		isset($data['vvi_AdjustedTradePrice_MileageAndCondition'])
		&& $this->vvi_AdjustedTradePrice_MileageAndCondition = $data['vvi_AdjustedTradePrice_MileageAndCondition'];

		isset($data['vvi_adjustedRetailPrice'])
		&& $this->vvi_adjustedRetailPrice = $data['vvi_adjustedRetailPrice'];

		isset($data['vvi_adjustedTradePrice'])
		&& $this->vvi_adjustedTradePrice = $data['vvi_adjustedTradePrice'];

		isset($data['vvi_estimatedRetailPrice'])
		&& $this->vvi_estimatedRetailPrice = $data['vvi_estimatedRetailPrice'];

		isset($data['vvi_estimatedTradePrice'])
		&& $this->vvi_estimatedTradePrice = $data['vvi_estimatedTradePrice'];

		isset($data['vvi_estimatedCostPrice'])
		&& $this->vvi_estimatedCostPrice = $data['vvi_estimatedCostPrice'];

		isset($data['vvi_adjustedEstimatedRetailPrice_mileageAndCondition'])
		&& $this->vvi_adjustedEstimatedRetailPrice_mileageAndCondition = $data['vvi_adjustedEstimatedRetailPrice_mileageAndCondition'];

		isset($data['vvi_adjustedEstimatedTradePrice_mileageAndCondition'])
		&& $this->vvi_adjustedEstimatedTradePrice_mileageAndCondition = $data['vvi_adjustedEstimatedTradePrice_mileageAndCondition'];

		isset($data['vvi_adjustedEstimatedcostprice_mileageandcondition'])
		&& $this->vvi_adjustedEstimatedcostprice_mileageandcondition = $data['vvi_adjustedEstimatedcostprice_mileageandcondition'];

		isset($data['vvi_adjustedEstimatedRetailPrice'])
		&& $this->vvi_adjustedEstimatedRetailPrice = $data['vvi_adjustedEstimatedRetailPrice'];

		isset($data['vvi_adjustedEstimatedTradePrice'])
		&& $this->vvi_adjustedEstimatedTradePrice = $data['vvi_adjustedEstimatedTradePrice'];

		isset($data['vvi_adjustedEstimatedCostPrice'])
		&& $this->vvi_adjustedEstimatedCostPrice = $data['vvi_adjustedEstimatedCostPrice'];

		isset($data['vvi_resultCode'])
		&& $this->vvi_resultCode = $data['vvi_resultCode'];

        isset($data['hpi_type'])
        && $this->hpi_type = $data['hpi_type'];

		/* ------------------------------------ Error codes ------------------------------------ */
		isset($data['error_code'])
		&& $this->error_code = $data['error_code'];



	}


}















































