<?php
namespace Valuation\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A valuation.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="stock_valuation")
 */
class Valuation
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = 'Build';
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = 'Update';
	/**
	 * Post insert action must be called after new entity is flushed to database.
	 */
	const HAVE_POST_INSERT = true;
	/**
	 * Handle as a job queue for mobile devices.
	 */
	const JOB_QUEUE = true;


	/*
	 * Department options.
	 */
	const DEPARTMENT_NEW  = 'New';
	const DEPARTMENT_USED = 'Used';



	/* ------------------------------------ Identification ------------------------------------ */
	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=12, nullable=true, name="valuation_number");
	 */
	protected $valuationNumber;


	/* ------------------------------------ Ownership ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\Stock\Entity\Stock", cascade={"all"})
	 * @ORM\JoinColumn(nullable=false, name="stock_id")
	 **/
	protected $stock;

	/**
	 * @ORM\ManyToOne(targetEntity="\Valuation\Entity\XmlRpc", cascade={"all"})
	 * @ORM\JoinColumn(nullable=true, name="xmlrpc_auth_id")
	 **/
	protected $xmlRpcClient;


	/* ------------------------------------ History ------------------------------------ */
	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=false, name="created_by_profile_id")
	 **/
	protected $createdBy;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=true, name="valuated_by_profile_id")
	 **/
	protected $valuatedBy;

	/**
	 * @ORM\OneToMany(targetEntity="ValuationValuators", mappedBy="valuation", cascade={"all"}, fetch="EXTRA_LAZY")
	 **/
	protected $valuators;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=true, name="sales_profile_id")
	 **/
	protected $salesProfile;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=true, name="manager_profile_id")
	 **/
	protected $managerProfile;

	/**
	 * @ORM\Column(type="datetime", nullable=true, name="sent_to_sales");
	 */
	protected $sentToSales;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="sales_force_item_id")
	 */
	protected $sfItemId;


// 	/* ------------------------------------ Customer ------------------------------------ */
	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="first_name")
	 */
	protected $firstName;

	/**
	 * @ORM\Column(type="string", length=50, nullable=true, name="family_name")
	 */
	protected $familyName;

	/**
	 * @ORM\Column(type="string", length=13, nullable=true, name="id_number")
	 */
	protected $idNumber;

	/**
	 * @ORM\Column(type="string", length=20, nullable=true)
	 */
	protected $mobile;

	/**
	 * @ORM\Column(type="string", length=255, nullable=true)
	 */
	protected $email;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="department");
	 */
	protected $department;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Region")
	 * @ORM\JoinColumn(nullable=true, name="lib_region_id")
	 **/
	protected $region;

	/**
	 * @ORM\ManyToOne(targetEntity="\Location\Entity\Town")
	 * @ORM\JoinColumn(nullable=true, name="lib_city_id")
	 **/
	protected $city;

	/**
	 * @ORM\Column(type="string", length=150, nullable=true)
	 */
	protected $street;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=true, name="required_price", options={"unsigned"=true});
	 */
	protected $requiredPrice = 0.0;

	/**
	 * @ORM\Column(type="boolean", nullable=true);
	 */
	protected $sighted = true;

    /**
     *  @ORM\Column(nullable=true, type="boolean", name="license_disc_expired");
     */
    protected $licenseDiscExpired = true;

	/**
	 * @ORM\Column(type="boolean", nullable=true, name="trade_retail_requested");
	 */
	protected $tradeRetailRequested;


	/* ------------------------------------ Pricing ------------------------------------ */
	/**
	 * @ORM\Column(type="text", name="sales_comments")
	 */
	protected $salesComments = '';

    /**
     * @ORM\Column(type="text", name="customer_comments")
     */
    protected $customerComments = '';

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="amount_offered", options={"unsigned"=true});
	 */
	protected $amountOffered = 0.0;

    /**
     * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="bank_settlement_amount", options={"unsigned"=true});
     */
    protected $bankSettlement = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="over_allowance", options={"unsigned"=true});
	 */
	protected $overAllowance = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="stand_in_value", options={"unsigned"=true});
	 */
	protected $standInValue = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="projected_retail", options={"unsigned"=true});
	 */
	protected $projectedRetail = 0.0;

	/**
	 * @ORM\Column(type="decimal", scale=2, precision=11, nullable=false, name="planned_margin", options={"unsigned"=true});
	 */
	protected $plannedMargin = 0.0;


    /**
     * @ORM\Column(type="string", length=150, nullable=true, name="customer_address_street")
     */
    protected $customerAddressStreet;

    /**
     * @ORM\Column(type="string", length=150, nullable=true, name="customer_address_street_name")
     */
    protected $customerAddressStreetName;

    /**
     * @ORM\Column(type="string", length=150, nullable=true, name="customer_address_suburb")
     */
    protected $customerAddressSuburb;

    /**
     * @ORM\Column(type="string", length=150, nullable=true, name="customer_address_city")
     */
    protected $customerAddressCity;

    /**
     * @ORM\Column(type="string", length=150, nullable=true, name="customer_address_postal_code")
     */
    protected $customerAddressPostalCode;

    /**
     * @ORM\ManyToOne(targetEntity="\Valuation\Entity\SendToStockFrom")
     * @ORM\JoinColumn(nullable=true, name="send_to_stock_from_id")
     **/
    protected $sendToStockFrom;

    /**
     * @ORM\ManyToOne(targetEntity="\Valuation\Entity\DealNotDoneSelection")
     * @ORM\JoinColumn(nullable=true, name="deal_not_done_selection_id")
     **/
    protected $dealNotDoneSelection;

    /**
     * @ORM\Column(type="text", name="deal_not_done_reason", nullable=true)
     */
    protected $dealNotDoneReason;


	/* ------------------------------------ Tracking ------------------------------------ */
	/**
	 * @ORM\Column(type="smallint", name="queue_status");
	 */
	protected $queueStatus = 0;

	/**
	 * @ORM\Column(nullable=true, type="string", length=25, nullable=true, name="internal_status");
	 */
	protected $internalState;

	/**
	 * @ORM\Column(type="string", length=25, nullable=true, name="previous_status");
	 */
	protected $previousState;

	/**
	 * @ORM\Column(nullable=true, type="string", length=25, name="status");
	 */
	protected $jobState;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;

	/**
	 * @ORM\Column(type="boolean",  name="is_public");
	 */
	protected $isPublic = false;

    /**
     * @ORM\Column(type="boolean",  name="is_public_customer");
     */
    protected $isPublicCustomer = false;

	/**
	 * Initialize collections.
	 */
	public function __construct()
	{
		$this->valuators = new \Doctrine\Common\Collections\ArrayCollection();
	}

	/**
	 * Add a new Valuator to this Stock Valuation Item.
	 * @param \User\Entity\Profile $profile
	 * @return \Valuation\Entity\Valuation
	 */
	public function addValuator(\User\Entity\Profile $profile)
	{
		$this->valuators[] = $profile;
		return $this;
	}



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		if (!\Utility\Registry::get('IsDeviceApiCall', false))
		{
			$this->createdBy = \Utility\Registry::isAuthenticated()
				? \Utility\Registry::resolveProfileContext($this->createdBy)
				: \Utility\Registry::getEntityManager()
					->getReference('\User\Entity\Profile', 1);
		}
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
		('New Valuation' != $this->jobState)
			&& 0 < $this->queueStatus
			&& $this->queueStatus = 0;
		if ('Price Guide' == $this->jobState
			&& is_null($this->stock->loadedOnPriceGuide))
		{
			$this->stock->loadedOnPriceGuide = new \DateTime("now");
		}
		if (!is_null($this->salesProfile) && is_null($this->sentToSales))
		{
			$this->sentToSales = new \DateTime("now");
		}
	}

	/**
	 * Automatically called from DataBin if HAVE_POST_INSERT constant is set on entity.
	 * @return boolean
	 */
	public function postInsert()
	{
		if (is_null($this->id) || !is_numeric($this->id))
		{
			return false;
		}
		$this->valuationNumber = 'V' . str_pad($this->id, 7, '0', STR_PAD_LEFT);
		return true;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['valuationNumber']))
			&& $data['valuationNumber'] = $this->valuationNumber;
		($includeAll || isset($intersect['stock']))
			&& $data['stock'] = (in_array('stock', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->stock)
				? (!$showIdentifiers || in_array('stock', $expand) ? $this->stock->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->stock->id)
				: null;
		($includeAll || isset($intersect['xmlRpcClient']))
			&& $data['xmlRpcClient'] = (in_array('xmlRpcClient', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->xmlRpcClient)
				? (!$showIdentifiers || in_array('xmlRpcClient', $expand) ? $this->xmlRpcClient->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->xmlRpcClient->id)
				: null;
		($includeAll || isset($intersect['createdBy']))
			&& $data['createdBy'] = (in_array('createdBy', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->createdBy)
				? (!$showIdentifiers || in_array('createdBy', $expand) ? $this->createdBy->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->createdBy->id)
				: null;
		($includeAll || isset($intersect['valuatedBy']))
			&& $data['valuatedBy'] = (in_array('valuatedBy', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->valuatedBy)
				? (!$showIdentifiers || in_array('valuatedBy', $expand) ? $this->valuatedBy->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->valuatedBy->id)
				: null;
		($includeAll || isset($intersect['valuators']))
			&& $data['valuators'] = (in_array('valuators', $expand) || $expandAll || $showIdentifiers)
										&& !is_null($this->valuators)
				? $this->valuatorsToArray($showIdentifiers)
				: null;
		($includeAll || isset($intersect['salesProfile']))
			&& $data['salesProfile'] = (in_array('salesProfile', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->salesProfile)
				? (!$showIdentifiers || in_array('salesProfile', $expand) ? $this->salesProfile->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->salesProfile->id)
				: null;
		($includeAll || isset($intersect['managerProfile']))
			&& $data['managerProfile'] = (in_array('managerProfile', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->managerProfile)
				? (!$showIdentifiers || in_array('managerProfile', $expand) ? $this->managerProfile->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->managerProfile->id)
				: null;
		($includeAll || isset($intersect['firstName']))
			&& $data['firstName'] = $this->firstName;
		($includeAll || isset($intersect['familyName']))
			&& $data['familyName'] = $this->familyName;
		($includeAll || isset($intersect['idNumber']))
			&& $data['idNumber'] = $this->idNumber;
		($includeAll || isset($intersect['mobile']))
			&& $data['mobile'] = $this->mobile;
		($includeAll || isset($intersect['email']))
			&& $data['email'] = $this->email;
		($includeAll || isset($intersect['department']))
			&& $data['department'] = $this->department;
		($includeAll || isset($intersect['region']))
			&& $data['region'] = (in_array('region', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->region)
				? (!$showIdentifiers || in_array('region', $expand) ? $this->region->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->region->id)
				: null;
		($includeAll || isset($intersect['city']))
			&& $data['city'] = (in_array('city', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->city)
				? (!$showIdentifiers || in_array('city', $expand) ? $this->city->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->city->id)
				: null;
		($includeAll || isset($intersect['street']))
			&& $data['street'] = $this->street;
		($includeAll || isset($intersect['sighted']))
			&& $data['sighted'] = $this->sighted;
        ($includeAll || isset($intersect['licenseDiscExpired']))
            && $data['licenseDiscExpired'] = $this->licenseDiscExpired;
		($includeAll || isset($intersect['requiredPrice']))
			&& $data['requiredPrice'] = $this->requiredPrice;
		($includeAll || isset($intersect['amountOffered']))
			&& $data['amountOffered'] = $this->amountOffered;
        ($includeAll || isset($intersect['bankSettlement']))
        && $data['bankSettlement'] = $this->bankSettlement;
		($includeAll || isset($intersect['salesComments']))
			&& $data['salesComments'] = $this->salesComments;

        ($includeAll || isset($intersect['customerComments']))
        && $data['customerComments'] = $this->customerComments;

        ($includeAll || isset($intersect['customerAddressStreet']))
        && $data['customerAddressStreet'] = $this->customerAddressStreet;
        ($includeAll || isset($intersect['customerAddressStreetName']))
        && $data['customerAddressStreetName'] = $this->customerAddressStreetName;
        ($includeAll || isset($intersect['customerAddressSuburb']))
        && $data['customerAddressSuburb'] = $this->customerAddressSuburb;
        ($includeAll || isset($intersect['customerAddressCity']))
        && $data['customerAddressCity'] = $this->customerAddressCity;
        ($includeAll || isset($intersect['customerAddressPostalCode']))
        && $data['customerAddressPostalCode'] = $this->customerAddressPostalCode;


        ($includeAll || isset($intersect['sendToStockFrom']))
        && $data['sendToStockFrom'] = (in_array('sendToStockFrom', $expand) || $expandAll || $showIdentifiers)
        && !is_null($this->sendToStockFrom)
            ? (!$showIdentifiers || in_array('sendToStockFrom', $expand) ? $this->sendToStockFrom->toArray(
                $expand, $intersect, $showIdentifiers, ($expandAll - 1)
            ) : $this->sendToStockFrom->id)
            : null;

        ($includeAll || isset($intersect['dealNotDoneSelection']))
        && $data['dealNotDoneSelection'] = (in_array('dealNotDoneSelection', $expand) || $expandAll || $showIdentifiers)
        && !is_null($this->dealNotDoneSelection)
            ? (!$showIdentifiers || in_array('dealNotDoneSelection', $expand) ? $this->dealNotDoneSelection->toArray(
                $expand, $intersect, $showIdentifiers, ($expandAll - 1)
            ) : $this->dealNotDoneSelection->id)
            : null;

        ($includeAll || isset($intersect['dealNotDoneReason']))
        && $data['dealNotDoneReason'] = $this->dealNotDoneReason;

		($includeAll || isset($intersect['overAllowance']))
			&& $data['overAllowance'] = $this->overAllowance;
		($includeAll || isset($intersect['standInValue']))
			&& $data['standInValue'] = $this->standInValue;
		($includeAll || isset($intersect['projectedRetail']))
			&& $data['projectedRetail'] = $this->projectedRetail;
		($includeAll || isset($intersect['plannedMargin']))
			&& $data['plannedMargin'] = $this->plannedMargin;
		($includeAll || isset($intersect['queueStatus']))
			&& $data['queueStatus'] = $this->queueStatus;
		($includeAll || isset($intersect['jobState']))
			&& $data['jobState'] = $this->jobState;

		($includeAll || isset($intersect['previousState']))
		&& $data['previousState'] = $this->previousState;
        ($includeAll || isset($intersect['isPublicCustomer']))
        && $data['isPublicCustomer'] = $this->isPublicCustomer;

		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Internal utility to change valuators collection into array.
	 * @param boolean $showIdentifiers
	 * @return array
	 */
	protected function valuatorsToArray($showIdentifiers = false)
	{
		$data = array();
		$iterator = $this->valuators->getIterator();
		foreach ($iterator as $valuator)
		{
			$data[] = $showIdentifiers
				? array('id' => $valuator->profile->id)
				: $valuator->toArray();
		}
		return $data;
	}

	/**
	 * Convert the object to an array for queue synchronization.
	 * @return array
	 */
	public function toQueueArray()
	{
		return array(
				'id'           => $this->id,
				'created'      => $this->created->format('Y-m-d H:i'),
				'created_by'   => $this->createdBy->firstName . ' ' . $this->createdBy->familyName,
				'reg_number'   => $this->stock->registrationNumber,
				'make'         => $this->stock->type->model->make->name,
				'model'        => $this->stock->type->model->name,
				'type'         => $this->stock->type->name,
				'queue_status' => $this->queueStatus
		);
	}

	/**
	 * Convert the object to an array for synchronization.
	 * @return array
	 */
	public function toSynchArray()
	{
		return array(
				'id'         => $this->id,
				'created'    => $this->created->format(
						\Utility\Registry::getConfigParam('DateTimeFormat')
						),
				'created_by' => $this->createdBy->firstName . ' ' . $this->createdBy->familyName,
				'reg_number' => $this->stock->registrationNumber,
				'make'       => $this->stock->type->model->make->name,
				'model'      => $this->stock->type->model->name,
				'type'       => $this->stock->type->name
		);
	}

	/**
	 * Query to use for synchronization pull.
	 * @return array
	 */
	public static function getSynchQuery()
	{
		return    'SELECT valuation '
				. 'FROM \Valuation\Entity\Valuation valuation '
				. 'JOIN valuation.valuators valuator '
				. 'WHERE (valuation.queueStatus = 1 AND IDENTITY(valuator.profile) = '
					. \Utility\Registry::getAuthParam('id') . ')'
				. ' OR (valuation.queueStatus = 2 AND IDENTITY(valuation.valuatedBy) = '
					. \Utility\Registry::getAuthParam('id') . ')';
	}

	/**
	 * Claim job queue item.
	 * @param \Doctrine\ORM\EntityManager $em
	 * @return boolean
	 */
	public function claimQueueItem(\Doctrine\ORM\EntityManager $em, $profileId = false)
	{
		$profileId = false !== $profileId
			? $profileId
			: \Utility\Registry::getAuthParam('id');
		if (2 == $this->queueStatus && $profileId != $this->valuatedBy->id)
		{
			return false;
		}
		$this->queueStatus = 2;
		$this->valuatedBy = $em->getReference(
				'\User\Entity\Profile',
				$profileId
				);
		$em->flush();
		return true;
	}

	/**
	 * Unclaim job queue item.
	 * @param \Doctrine\ORM\EntityManager $em
	 * @return boolean
	 */
	public function unclaimQueueItem(\Doctrine\ORM\EntityManager $em)
	{
		$this->queueStatus = 1;
		$this->valuatedBy  = null;
		$em->flush();
		return true;
	}

	/**
	 * Complete claimed job queue item.
	 * @param \Doctrine\ORM\EntityManager $em
	 */
	public function completeQueueItem(\Doctrine\ORM\EntityManager $em)
	{
		$iterator = $this->valuators->getIterator();
		foreach ($iterator as $val)
		{
			$em->remove($val);
		}
		$em->flush();
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['stock'])
			&& $this->stock = $data['stock'];
		isset($data['xmlRpcClient'])
			&& $this->xmlRpcClient = $data['xmlRpcClient'];
		isset($data['createdBy'])
			&& $this->createdBy = $data['createdBy'];
		isset($data['valuatedBy'])
			&& $this->valuatedBy = $data['valuatedBy'];
		isset($data['salesProfile'])
			&& $this->salesProfile = $data['salesProfile'];
		isset($data['firstName'])
			&& $this->firstName = $data['firstName'];
		isset($data['familyName'])
			&& $this->familyName = $data['familyName'];
		isset($data['idNumber'])
			&& $this->idNumber = $data['idNumber'];
		isset($data['mobile'])
			&& $this->mobile = $data['mobile'];
		isset($data['email'])
			&& $this->email = $data['email'];
		isset($data['department'])
			&& $this->department = $data['department'];
		isset($data['region'])
			&& $this->region = $data['region'];
		isset($data['city'])
			&& $this->city = $data['city'];
		isset($data['street'])
			&& $this->street = $data['street'];
		isset($data['sighted'])
			&& $this->sighted = $data['sighted'];
        isset($data['licenseDiscExpired'])
            && $this->licenseDiscExpired = $data['licenseDiscExpired'];
		isset($data['requiredPrice'])
			&& $this->requiredPrice = $data['requiredPrice'];
		isset($data['salesComments'])
			&& $this->salesComments = $data['salesComments'];

        isset($data['customerComments'])
        && $this->customerComments = $data['customerComments'];

        isset($data['customerAddressStreet'])
        && $this->customerAddressStreet = $data['customerAddressStreet'];
        isset($data['customerAddressStreetName'])
        && $this->customerAddressStreetName = $data['customerAddressStreetName'];
        isset($data['customerAddressSuburb'])
        && $this->customerAddressSuburb = $data['customerAddressSuburb'];
        isset($data['customerAddressCity'])
        && $this->customerAddressCity = $data['customerAddressCity'];
        isset($data['customerAddressPostalCode'])
        && $this->customerAddressPostalCode = $data['customerAddressPostalCode'];

        isset($data['sendToStockFrom'])
        && $this->sendToStockFrom = $data['sendToStockFrom'];
        isset($data['dealNotDoneSelection'])
        && $this->dealNotDoneSelection = $data['dealNotDoneSelection'];
        isset($data['dealNotDoneReason'])
        && $this->dealNotDoneReason = $data['dealNotDoneReason'];

		isset($data['amountOffered'])
			&& $this->amountOffered = $data['amountOffered'];
        isset($data['bankSettlement'])
        && $this->bankSettlement = $data['bankSettlement'];
		isset($data['overAllowance'])
			&& $this->overAllowance = $data['overAllowance'];
		isset($data['standInValue'])
			&& $this->standInValue = $data['standInValue'];
		isset($data['projectedRetail'])
			&& $this->projectedRetail = $data['projectedRetail'];
		isset($data['plannedMargin'])
			&& $this->plannedMargin = $data['plannedMargin'];
		isset($data['queueStatus'])
			&& $this->queueStatus = $data['queueStatus'];
		if (isset($data['valuators']))
		{
			$em = \Utility\Registry::getEntityManager();
			$newVals = array();
			foreach ($data['valuators'] as $val)
			{
				$newVals[$val['id']] = $val['id'];
			}
			$currentVals = array();
			$iterator = $this->valuators->getIterator();
			foreach ($iterator as $val)
			{
				$currentVals[$val->id] = $val->id;
				!isset($newVals[$val->id])
					&& $em->remove($val);
			}
			$newVals = array();
			foreach ($data['valuators'] as $val)
			{
				if (!isset($currentVals[$val['id']]))
				{
					$valuator = new ValuationValuators();
					$valuator->profile = $em->getReference('\User\Entity\Profile', $val['id']);
					$valuator->valuation = $this;
					$this->valuators->add($valuator);
				}
			}
		}
	}



}

