<?php
namespace Valuation\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * An xml-rpc request log entry.
 *
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="xmlrpc_callback_log")
 */
class XmlRpcCallbackLog
{

	/**
	 * Cannot archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;



	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\ManyToOne(targetEntity="\Valuation\Entity\XmlRpc", cascade={"all"})
	 * @ORM\JoinColumn(nullable=false, name="xmlrpc_auth_id")
	 **/
	protected $xmlRpcClient;

	/**
	 * @ORM\ManyToOne(targetEntity="Valuation")
	 * @ORM\JoinColumn(nullable=true, name="stock_valuation_id", referencedColumnName="id")
	 **/
	protected $valuation;

	/**
	 * @ORM\Column(type="string", length=250, nullable=false, name="callback_url");
	 */
	protected $callbackUrl;

	/**
	 * @ORM\Column(type="string", length=50, nullable=false, name="method_name");
	 */
	protected $methodName;

	/**
	 * @ORM\Column(type="text", nullable=true, name="packet");
	 */
	protected $packet;

	/**
	 * @ORM\Column(type="text", nullable=true, name="response");
	 */
	protected $response;

	/**
	 * @ORM\Column(type="string", length=50, nullable=false, name="status");
	 */
	protected $status;

	/**
	 * @ORM\Column(type="integer", nullable=false, name="attempts");
	 */
	protected $attempts = 1;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;



	/**
	 * Magic getter to expose protected properties.
	 *
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 *
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @param boolean $showIdentifiers
	 * @param integer $expandAll
	 * @return array
	 */
	public function toArray(
			array $expand = array(), array $intersect = array(),
			$showIdentifiers = false, $expandAll = 0
			)
	{
		$intersect = array_flip($intersect);
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['xmlRpcClient']))
			&& $data['xmlRpcClient'] = (in_array('xmlRpcClient', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->xmlRpcClient)
				? (!$showIdentifiers || in_array('xmlRpcClient', $expand) ? $this->xmlRpcClient->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->xmlRpcClient->id)
				: null;
		($includeAll || isset($intersect['valuation']))
			&& $data['valuation'] = (in_array('valuation', $expand) || $expandAll || $showIdentifiers)
									&& !is_null($this->valuation)
				? (!$showIdentifiers || in_array('valuation', $expand) ? $this->valuation->toArray(
						$expand, $intersect, $showIdentifiers, ($expandAll - 1)
						) : $this->valuation->id)
				: null;
		($includeAll || isset($intersect['callbackUrl']))
			&& $data['callbackUrl'] = $this->callbackUrl;
		($includeAll || isset($intersect['methodName']))
			&& $data['methodName'] = $this->methodName;
		($includeAll || isset($intersect['packet']))
			&& $data['packet'] = $this->packet;
		($includeAll || isset($intersect['response']))
			&& $data['response'] = $this->response;
		($includeAll || isset($intersect['status']))
			&& $data['status'] = $this->status;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['xmlRpcClient'])
			&& $this->xmlRpcClient = $data['xmlRpcClient'];
		isset($data['valuation'])
			&& $this->valuation = $data['valuation'];
		isset($data['callbackUrl'])
			&& $this->callbackUrl = $data['callbackUrl'];
		isset($data['methodName'])
			&& $this->methodName = $data['methodName'];
		isset($data['packet'])
			&& $this->packet = is_array($data['packet'])
				? serialize($data['packet'])
				: $data['packet'];
		isset($data['response'])
			&& $this->response = is_array($data['response'])
				? serialize($data['response'])
				: $data['response'];
		isset($data['status'])
			&& $this->status = $data['status'];
	}



}

