<?php
namespace Valuation\Report;



/**
 * Stock Contyrol Report.
 * @author andre.fourie
 */
class ValuationGrid extends \Utility\Service\Report
{



	/**
	 * @var string
	 */
	protected $_title  = 'Valuation Data Export';
	/**
	 * @var string
	 */
	protected $_subject = 'Valuations';
	/**
	 * @var string
	 */
	protected $_description = 'Export of valuations.';
	/**
	 * @var array
	 */
	protected $_notes = array(
			'Confidential information, generated using Bid 4 Cars, for more information visit bid4cars.com.na'
			);
	/**
	 * @var array
	 */
	protected $_headers = array(
			'Dealer',
			'Group',
			'Division',
			'Reference Number',
			'Registration Number',
			'VIN Number',
			'Make',
			'Model',
			'Type',
			'MM Code',
			'Year',
			'Main Exterior Colour',
			'KM',
			'Trade Price',
			'Recon costs',
			'Price Offered to Sales',
			'% of Trade',
			'Date loaded on price guide',
			'Number of price guide offers',
			'Highest price guide offer',
			'Highest price Dealer Name(s)',
			'Status',
			'Sales Person',
			'Manager'
			);
	/**
	 * @var array
	 */
	protected $_fields = array(
			'company_name',
			'companyGroup_name',
			'groupDivision_name',
			'stock_referenceNumber',
			'stock_registrationNumber',
			'stock_vinNumber',
			'make_name',
			'model_name',
			'type_name',
			'type_mmCode',
			'vehicleYear_name',
			'exteriorColour_name',
			'stock_km',
			'stock_tradePrice',
			'stock_damageTotal',
			'valuation_amountOffered',
			'valuation_updated',
			'stock_loadedOnPriceGuide',
			'stock_numberOfOffers',
			'stock_highestOffer',
			'higestoffernames',
			'valuation_jobState',
			'salesProfile_fullName',
			'managerProfile_fullName'
			);
	/**
	 * @var array
	 */
	protected $_totals = array(
			'stock_tradePrice',
            'stock_listPrice',
			'stock_damageTotal',
			'stock_retailPrice',

			'stock_highestOffer'
			);
	/**
	 * @var array
	 */
	protected $_currencyFields = array(
			'stock_tradePrice',
            'stock_listPrice',
			'stock_damageTotal',
			'stock_retailPrice',
			'stock_highestOffer',
			'valuation_amountOffered'
			);



	/**
	 * Build the dataset.
	 */
	public function build()
	{
		#-> Data passed by DataBin Grid functionality.
		$this->em->clear();
		ini_set('memory_limit','512M');
		foreach ($this->_data as $rowId => $rowData)
		{
			$this->_data[$rowId]['salesProfile_fullName'] = isset($rowData['salesProfile_firstName'])
													&& !empty($rowData['salesProfile_firstName'])
				? $rowData['salesProfile_firstName'] . ' ' . $rowData['salesProfile_familyName']
				: '';
			$this->_data[$rowId]['managerProfile_fullName'] = isset($rowData['managerProfile_firstName'])
													&& !empty($rowData['managerProfile_firstName'])
				? $rowData['managerProfile_firstName'] . ' ' . $rowData['managerProfile_familyName']
				: '';
			if (is_null($rowData['priceGuide_id']))
			{
				$this->_data[$rowId]['higestoffernames'] = '';
				continue;
			}
			$query = 'SELECT priceGuideOffer, company '
					. 'FROM \PriceGuide\Entity\Offer priceGuideOffer '
					. 'JOIN priceGuideOffer.company company '
					. 'WHERE IDENTITY(priceGuideOffer.priceGuideStock) = :priceGuideId'
					. ' AND priceGuideOffer.amount = :highestOffer';
			$query = $this->em->createQuery($query);
			$query->setParameter('priceGuideId', $rowData['priceGuide_id']);
			$query->setParameter('highestOffer', $rowData['stock_highestOffer']);
			$resultx = $query->getArrayResult();
			$this->em->clear();
			if (empty($resultx))
			{
				$this->_data[$rowId]['higestoffernames'] = '';
				continue;
			}
			$dealers = array();
			foreach($resultx as $itemx)
			{
				$dealers[] = $itemx['company']['name'];
			}
			$this->_data[$rowId]['higestoffernames'] = implode(', ', $dealers);

		}

		foreach ($this->_data as $rowId => $rowData)
		{
			if ($this->_data[$rowId]['stock_tradePrice'] == 0)
			{
				$this->_data[$rowId]['valuation_updated'] = '0 %';
			}
			else
			{
				$this->_data[$rowId]['valuation_updated'] = round(($this->_data[$rowId]['valuation_amountOffered'] * 100) / $this->_data[$rowId]['stock_tradePrice']) . ' %' ;
			}
		}

	}

}
