<?php
namespace Workspace\Controller;



class DeviceWorkspaceController extends \Zend\Mvc\Controller\AbstractActionController
{

	/**
	 * @var \Zend\Session\Container
	 */
	protected $session;
	/**
	 * @var array
	 */
	protected $request;
	/**
	 * @var array
	 */
	protected $response;
	/**
	 * @var \Zend\ServiceManager\ServiceManager
	 */
	protected $sm;

	protected $em;
	/**
	 * @var boolean
	 */
	protected $authenticated = false;
	/**
	 * @var \User\Service\Authentication
	 */
	protected $authDetails;



	/**
	 * Retrieve workspace manager.
	 */
	public function initPost($initSession = true, $tokenParam = 'authToken')
	{
		defined('APPLICATION')
			|| define('APPLICATION', 'Mobile');
		$this->request = $_POST;
		$this->response = array();
		if ($initSession)
		{
			if (!isset($this->request[$tokenParam]) || empty($this->request[$tokenParam]))
			{
				return false;
			}
			try
			{
				\Utility\Registry::setSession($this->request[$tokenParam]);
				$this->authenticated = true;
				$this->authDetails = \Utility\Registry::getAuthData();
			}
			catch (\Exception $e)
			{
				return false;
			}
		}
		\Utility\Registry::setServiceManager($this->serviceLocator);
		\Utility\Registry::addDynamicConfig(
				\Utility\Registry::getEntityManager()
					->getRepository('Config\Entity\Config')
					->find(1)
					->toArray()
				);
		return true;
	}

	/**
	 * Convenience method to extract parameters from request packet.
	 * @param string $paramName
	 * @param unknown $default
	 * @return unknown
	 */
	protected function reqParam($paramName, $default = null)
	{
		return isset($this->request[$paramName])
			? $this->request[$paramName]
			: $default;
	}

	/**
	 * Convenience method to extract parameters from a nested request packet.
	 * @param string $root
	 * @param string $paramName
	 * @param unknown $default
	 * @return unknown
	 */
	protected function reqNestedParam($root, $paramName, $default = null)
	{
		return isset($this->request[$root][$paramName])
			? $this->request[$root][$paramName]
			: $default;
	}

	/**
	 * Convenience method to extract parameters from a nested request packet.
	 * @param string $root
	 * @param string $paramName
	 * @param string $entityName
	 * @param unknown $default
	 * @return unknown
	 */
	protected function reqNestedParamRef($root, $paramName, $entityName, $default = null)
	{
		return isset($this->request[$root][$paramName])
				&& !is_null($this->request[$root][$paramName])
				&& 0 != $this->request[$root][$paramName]
			? $this->em->getReference($entityName, $this->request[$root][$paramName])
			: $default;
	}



	/**
	 * Display list of tasks for specified workflows.
	 * @see \Zend\Mvc\Controller\AbstractActionController::indexAction()
	 * @return \Zend\View\Model\JsonModel
	 */
	public function indexAction()
	{
		return new \Zend\View\Model\JsonModel(array(
				'login' => 'Requires an array with `email`, `pin`.',
				'get-dataset' => 'Requires `token`, `datasetName`, [`page`, `timeStart`, `timeEnd`]'
		));
	}


	/**
	 * Mobile handshake.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function pingAction()
	{
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
						'Task'         => 'Ping',
						'Status'       => 'Success',
						'StatusReason' => ''
				),
				'Data' => array(
						'ServerTime' => time()
				)
		));
	}


	/**
	 * Mobile device connected to push server.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function registerDeviceAction()
	{
		$this->initPost(true, 'ident');

		if ($this->authenticated)
		{
			\Utility\Registry::setAuthParam('pubid', $this->request['pubid']);
			$em = \Utility\Registry::getEntityManager();
			$session = $em->getRepository('\User\Entity\Session')
				->findOneBy(array('authToken' => $this->request['ident']));
			if (!is_null($session))
			{
				$session->pubId = $this->request['pubid'];
				$em->flush();
			}
		}
		/* error_log('User connected');
		$this->authenticated
			&& error_log('Email: ' . $this->authDetails['email']);
		error_log('pubid: ' . $this->request['pubid']);
		error_log('ident: ' . $this->request['ident']); */
		return new \Zend\View\Model\JsonModel(array('ApeReg' => 'Success'));
	}

	/**
	 * Mobile device disconnected from push server.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function deregisterDeviceAction()
	{
		$this->initPost(true, 'ident');

		if ($this->authenticated)
		{
			\Utility\Registry::setAuthParam('pubid', null);
			$em = \Utility\Registry::getEntityManager();
			$session = $em->getRepository('\User\Entity\Session')
				->findOneBy(array('authToken' => $this->request['ident']));
			if (!is_null($session))
			{
				$session->pubId = null;
				$em->flush();
			}
		}
		/* error_log('User disconnected');
		error_log('pubid: ' . $this->request['pubid']);
		error_log('ident: ' . $this->request['ident']); */
		return new \Zend\View\Model\JsonModel(array('ApeDereg' => 'Success'));
	}

	/**
	 * Broadcast a dataset change
	 * @return \Zend\View\Model\JsonModel
	 */
	public function notifyDeviceAction()
	{
		$this->initJson(false);

		$ds = isset($this->request['ds'])
			? $this->request['ds']
			: 'countries';
		$pubid = isset($this->request['pubid'])
			? $this->request['pubid']
			: false;
		//$response = \Utility\Comms\Ape::broadcast('synchv1', array('dataSet' => $ds), 'data', $pubid);
		return new \Zend\View\Model\JsonModel(array('Response' => $response));
	}

	/**
	 * Log debug data to db.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function logDebugAction()
	{
		$this->initPost(false);
		$em = \Utility\Registry::getEntityManager();
		$debugLog = new \Utility\Entity\DebugLog();
		$debugLog->userId      = $this->reqParam('userId', null);
		$debugLog->callName    = $this->reqParam('callName', null);
		$debugLog->params      = serialize($this->reqParam('params', array()));
		$debugLog->errorString = $this->reqParam('errorString', null);
		$em->persist($debugLog);
		$em->flush();
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
						'Task'         => 'LogDebug',
						'Status'       => 'Success',
						'StatusReason' => ''
				),
				'Data' => array()
		));
	}

	/**
	 * Device user login.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function loginAction()
	{
		$this->initPost(false);

		$input = new \Workspace\Utility\ServiceInput('ParamSet', array(
				'Login' => array(
						'email' => $this->request['email'],
						'pin'   => $this->request['pin']
				)
		));
		$serviceInput = new \Workspace\Utility\ServiceInput('ServiceInput', array(
				'data' => $input->pack()
		));
		$profileService = $this->serviceLocator->get('User.Service.Profile');
		$response = $profileService->deviceLogin($serviceInput->pack());
		if (isset($response['Data']['authToken']))
		{
			$em = \Utility\Registry::getEntityManager();
			$session = new \User\Entity\Session();
			$session->profile = $em->getReference('\User\Entity\Profile', $response['Data']['id']);
			$session->authToken = $response['Data']['authToken'];
			$em->persist($session);
			$em->flush();
		}
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array_merge(array('Task' => 'Login'), $response['Meta']),
				'Data' => $response['Data']
		));
	}

	/**
	 * Device user pin reset.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function resetPinAction()
	{
		$this->initPost(false);

		$input = new \Workspace\Utility\ServiceInput('ParamSet', array(
				'Login' => array(
						'email'  => isset($this->request['email'])
									? $this->request['email']
									: null,
						'mobile' => isset($this->request['mobile'])
									? $this->request['mobile']
									: null
				)
		));
		$serviceInput = new \Workspace\Utility\ServiceInput('ServiceInput', array(
				'data' => $input->pack()
		));
		$profileService = $this->serviceLocator->get('User.Service.Profile');
		$response = $profileService->deviceResetPin($serviceInput->pack());
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array_merge(array('Task' => 'ResetPin'), $response['Meta']),
				'Data' => $response['Data']
		));
	}

	/**
	 * Claim a job item from queue.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function claimJobAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'ClaimJob',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		if (!isset($this->request['datasetName'])
			|| !isset($this->request['id'])
			|| 0 >= $this->request['id'])
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'ClaimJob',
							'Status'       => 'Fail',
							'StatusReason' => 'Api call requires valid `datasetName` and `id` parameters.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}
		$ds = $this->request['datasetName'];
		$id = $this->request['id'];
		switch ($ds)
		{
			case 'valuations':
			default:
				$entityName = '\Valuation\Entity\Valuation';
				$service = new \Valuation\Service\Valuation();
				$service->setWorkflow(new \Valuation\Workflow());
				break;
		}
		$em = \Utility\Registry::getEntityManager();
		$record = $em->getRepository($entityName)
			->find($id);
		if (is_null($record) || 1 != $record->queueStatus)
		{
			if (!$record->claimQueueItem($em))
			{
				return new \Zend\View\Model\JsonModel(array(
						'Meta' => array(
								'Task'         => 'ClaimJob',
								'Status'       => 'Fail',
								'StatusReason' => 'Job not available.',
								'DatasetName'  => $ds
						),
						'Data' => array('id' => $record->id)
				));
			}
			else
			{
				return new \Zend\View\Model\JsonModel(array(
						'Meta' => array(
								'Task'         => 'ClaimJob',
								'Status'       => 'Success',
								'StatusReason' => '',
								'DatasetName'  => $ds
						),
						'Data' => $record->toArray(array('stock'), array(), 2)
				));
			}
		}
		$record->claimQueueItem($em);
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
						'Task'         => 'ClaimJob',
						'Status'       => 'Success',
						'StatusReason' => '',
						'DatasetName'  => $ds
				),
				'Data' => $record->toArray(array('stock'), array(), 2)
		));
	}

	/**
	 * Return a claimed job item from queue.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function unclaimJobAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'UnclaimJob',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		if (!isset($this->request['datasetName'])
			|| !isset($this->request['id'])
			|| 0 >= $this->request['id'])
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'ClaimJob',
							'Status'       => 'Fail',
							'StatusReason' => 'Api call requires valid `datasetName` and `id` parameters.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}
		$ds = $this->request['datasetName'];
		$id = $this->request['id'];
		switch ($ds)
		{
			case 'valuations':
			default:
				$entityName = '\Valuation\Entity\Valuation';
				$service = new \Valuation\Service\Valuation();
				$service->setWorkflow(new \Valuation\Workflow());
				break;
		}
		$em = \Utility\Registry::getEntityManager();
		$record = $em->getRepository($entityName)
			->find($id);
		if (is_null($record) || 2 != $record->queueStatus)
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'UnclaimJob',
							'Status'       => 'Fail',
							'StatusReason' => 'Job not claimed.',
							'DatasetName'  => $ds
					),
					'Data' => $record->toArray()
			));
		}
		try
		{
			$record->unclaimQueueItem($em);
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'UnclaimJob',
							'Status'       => 'Success',
							'StatusReason' => '',
							'DatasetName'  => $ds
					),
					'Data' => $record->toArray(array(), array(), false, 2)
			));
		}
		catch (\Exception $e)
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'UnclaimJob',
							'Status'       => 'Fail',
							'StatusReason' => 'Job owned by different user.',
							'DatasetName'  => $ds
					),
					'Data' => $record->toArray()
			));
		}
	}

	/**
	 * Synchronize critical valuation info.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function synchPhotoAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'SynchPhoto',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		$this->em = \Utility\Registry::getEntityManager();
		$params = json_decode($this->request['params'], true);
		$params = $params[0];
		$uvi    = $params['uvi'];
		$type   = $params['type'];
		$params['photo'] = base64_decode($params['photo']);
		$this->em = \Utility\Registry::getEntityManager();
		$stock = $this->em
			->getRepository('\Stock\Entity\Stock')
			->findOneBy(array('uvi' => $uvi));
		if ($stock)
		{
			#-> Write image to file.
			$filePath = \Utility\Registry::getConfigParam('ImagePath');
			$thumbPath = \Utility\Registry::getConfigParam('ThumbnailPath');
			file_put_contents($filePath . $type . $uvi . '.jpg', $params['photo']);
			$this->resizeImage($params['photo'], 'image/jpeg', $thumbPath . $type . $uvi . '.jpg');

			#-> Create image entry.
			$image = new \Utility\Entity\Image();
			$image->filename = $type . $uvi . '.jpg';
			$image->mimeType = 'image/jpeg';
			$this->em->persist($image);
			$this->em->flush();

			#-> Update stock entry.
			switch ($type)
			{
				case 'MAIN':
					$stock->mainImage = $image;
					break;
				case 'FRONT':
					$stock->frontImage = $image;
					break;
				case 'RIGHT':
					$stock->rightImage = $image;
					break;
				case 'LEFT':
					$stock->leftImage = $image;
					break;
				case 'BACK':
					$stock->backImage = $image;
					break;
				case 'ENGINE':
					$stock->engineImage = $image;
					break;
				case 'INTERIOR':
					$stock->interiorImage = $image;
					break;
				case 'NATIS':
					$stock->natisImage = $image;
					break;
			}
			$this->em->flush();
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'SynchPhoto',
							'Status'       => 'Success',
							'StatusReason' => 'Synchronized.'
					),
					'Data' => array(array('uvi' => $uvi, 'type' => $type))
			));
		}
		else
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'SynchPhoto',
							'Status'       => 'Error',
							'StatusReason' => 'Valuation with UVI ' . $uvi . ' not found.'
					),
					'Data' => array()
			));
		}
	}

	private function resizeImage($imgString, $mimeType, $destination)
	{
		switch ($mimeType)
		{
			case 'image/jpeg':
			case 'image/pjpeg':
				$fileExt = '.jpg';
				break;
			case 'image/gif':
				$fileExt = '.gif';
				break;
			case 'image/png':
				$fileExt = '.png';
				break;
			case 'image/bmp':
			case 'image/x-windows-bmp':
				$fileExt = '.bmp';
				break;
			default:
				return false;
		}
		/* $tmpFile = \Utility\Registry::getConfigParam('ImagePath')
						 . mt_rand(10000000, 99999999) . $fileExt; */
		$tmpFile = $destination;
		$bytesWritten = file_put_contents($tmpFile, $imgString);
		list($img_width, $img_height) = getimagesize($tmpFile);
		unlink($tmpFile);
		if (!$img_width || !$img_height)
		{
			return false;
		}
		$src_img = imagecreatefromstring($imgString);
		$scale = min(
				300 / $img_width,
				200 / $img_height
		);
		if ($scale >= 1)
		{
			file_put_contents($tmpFile, $imgString);
			return true;
		}
		$new_width = $img_width * $scale;
		$new_height = $img_height * $scale;
		$new_img = @imagecreatetruecolor($new_width, $new_height);
		switch ($mimeType)
		{
			case 'image/jpeg':
			case 'image/pjpeg':
				$write_image = 'imagejpeg';
				$image_quality = 95;
				break;
			case 'image/gif':
				@imagecolortransparent($new_img, @imagecolorallocate($new_img, 0, 0, 0));
		        $write_image = 'imagegif';
		        $image_quality = null;
		        break;
			case 'image/png':
				@imagecolortransparent($new_img, @imagecolorallocate($new_img, 0, 0, 0));
				@imagealphablending($new_img, false);
				@imagesavealpha($new_img, true);
				$write_image = 'imagepng';
				$image_quality = 9;
				break;
			case 'image/bmp':
			case 'image/x-windows-bmp':
				$write_image = 'imagebmp';
				$image_quality = null;
				break;
			default:
				return false;
		}
		$success = @imagecopyresampled(
				$new_img,
				$src_img,
				0, 0, 0, 0,
				$new_width,
				$new_height,
				$img_width,
				$img_height
		);
		if ($success)
		{
			$write_image($new_img, $tmpFile, $image_quality);
			@imagedestroy($src_img);
			@imagedestroy($new_img);
			return true;
		}
		@imagedestroy($src_img);
		@imagedestroy($new_img);
		return false;
	}

	/**
	 * Synchronize multiple valuations.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function synchValuationsAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'SynchValuations',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		$this->em = \Utility\Registry::getEntityManager();
		$input     = json_decode($this->request['params'], true);
		$responses = array();
		foreach ($input as $paramSet)
		{
			$response = $this->synchValuation($paramSet);
			$responses[] = 'Success' == $response['Meta']['Status']
				? $response['Data']
				: array('uvi' => null);
		}
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
						'Task'         => 'SynchValuations',
						'Status'       => 'Success',
						'StatusReason' => ''
				),
				'Data' => $responses
		));
	}

	/**
	 * Synchronize critical valuation info.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function synchValuationInfoAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'SynchValuationInfo',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		$this->em = \Utility\Registry::getEntityManager();
		$data = json_decode($this->request['params'], true);
		$response = $this->synchValuation($data);
		//$response['Data'] = array($response['Data']);
		return new \Zend\View\Model\JsonModel(
				$response
				);
	}

	/**
	 * Synchronize critical valuation info.
	 * @return \Zend\View\Model\JsonModel
	 */
	private function synchValuation($params)
	{
		$this->request['params'] = $params;
		$this->request['params']['vehicle_accessories'] = is_array($this->request['params']['vehicle_accessories'])
			? $this->request['params']['vehicle_accessories']
			: json_decode($this->request['params']['vehicle_accessories'], true);
		$this->request['params']['vehicle_damages'] = is_array($this->request['params']['vehicle_damages'])
			? $this->request['params']['vehicle_damages']
			: json_decode($this->request['params']['vehicle_damages'], true);
		$this->request['params']['vehicle_accessories'] = isset($this->request['params']['vehicle_accessories']['values'])
			? $this->request['params']['vehicle_accessories']['values']
			: $this->request['params']['vehicle_accessories'];
		$this->request['params']['vehicle_damages'] = isset($this->request['params']['vehicle_damages']['values'])
			? $this->request['params']['vehicle_damages']['values']
			: $this->request['params']['vehicle_damages'];

		if (is_null($this->em))
		{
			$this->em = \Utility\Registry::getEntityManager();
		}
		$valuationId = $this->reqNestedParam('params', 'server_id', false);
		$uvi = $this->reqNestedParam('params', 'valuation_id', false);
		$tt = $this->reqNestedParamRef('params', 'vehicle_type', '\Stock\Entity\Type', false);
		if (!$valuationId && !$uvi && !$tt)
		{
			return array(
					'Meta' => array(
							'Task'         => 'SynchValuation',
							'Status'       => 'Exception',
							'StatusReason' => 'Insufficient data provided.'
					),
					'Data' => array('uvi' => $uvi)
			);
		}
		$isNewEntry = false;
		$pgOverrideDoNotSend = false;
		if ($valuationId)
		{
			$valuation = $this->em
				->getRepository('\Valuation\Entity\Valuation')
				->find($valuationId);
			if (is_null($valuation))
			{
				$valuation = new \Valuation\Entity\Valuation();
				$stock = new \Stock\Entity\Stock();
	      		$valuation->stock = $stock;
	      		$isNewEntry = true;
			}
			else
			{
				$stock = $valuation->stock;
				if ('Price Guide' == $valuation->jobState || 'Price Guide' == $stock->jobState)
				{
					$pgOverrideDoNotSend = true;
				}
			}
		}
		else
		{
			$stock = $this->em
				->getRepository('\Stock\Entity\Stock')
				->findOneBy(array('uvi' => $uvi));
			if ($stock)
			{
				if (!is_null($stock->valuation))
				{
					$valuation = $this->em
						->getRepository('\Valuation\Entity\Valuation')
						->find($stock->valuation->id);
					if ('Price Guide' == $valuation->jobState || 'Price Guide' == $stock->jobState)
					{
						$pgOverrideDoNotSend = true;
					}
				}
				else
				{
					$valuation = new \Valuation\Entity\Valuation();
					$stock = new \Stock\Entity\Stock();
					$valuation->stock = $stock;
					$isNewEntry = true;
				}
			}
			else
			{
				$valuation = new \Valuation\Entity\Valuation();
				$stock = new \Stock\Entity\Stock();
	      		$valuation->stock = $stock;
	      		$isNewEntry = true;
			}
		}

		$iterator = $stock->damages->getIterator();
		foreach ($iterator as $dmg)
		{
			$this->em->remove($dmg);
		}
		$iterator = $stock->accessories->getIterator();
		foreach ($iterator as $acc)
		{
			$this->em->remove($acc);
		}
		$this->em->flush();


		$accessories = array();
		foreach ($this->reqNestedParam('params', 'vehicle_accessories', array()) as $accId)
		{
			$accessories[] = array('id' => $accId);
		}

		$tradePrice = $this->reqNestedParam('params', 'trade_price', null);
		$retailPrice = $this->reqNestedParam('params', 'retail_price', null);
        $listPrice = $this->reqNestedParam('params', 'list_price', null);
		/* try
		{
			if (is_null($tradePrice) || 0 == $tradePrice
				|| is_null($retailPrice) || 0 == $retailPrice)
			{
				$vehicleYear = $this->em->find(
						'\Stock\Entity\Year',
						$this->reqNestedParam('params', 'vehicle_year', null));
				$vehicleType = $this->em->find(
						'\Stock\Entity\Type',
						$this->reqNestedParam('params', 'vehicle_type', null));
				if (is_object($vehicleYear) && is_object($vehicleType))
				{
					if ($vehicleYear->name != date('Y'))
					{
						$tu = new \Utility\Comms\TransUnion();
						$tuRes = $tu->searchByMmCode($vehicleType->mmCode, $vehicleYear->name);
						\Utility\Debug::errorLog('TY Fetch on Synch', $tuRes);
						if (false != $tuRes)
						{
							$tradePrice = $tuRes['VehicleDetails'][0]['Value']['TradePrice'];
							$retailPrice = $tuRes['VehicleDetails'][0]['Value']['RetailPrice'];
						}
					}
				}
			}
		}
		catch (\Exception $e)
		{
			\Utility\Debug::errorLog('TY Fetch Error on Synch', $e->getMessage());
		} */

		$keys = $this->reqNestedParam('params', 'vehicle_spare_keys', null);
		if (null != $keys)
		{
			$keys = 'Yes' == $keys
				? true
				: false;
		}
		$stockData = array(
				'vehicleYear' => $this->reqNestedParamRef('params', 'vehicle_year', '\Stock\Entity\Year', null),
				'type' => $this->reqNestedParamRef('params', 'vehicle_type', '\Stock\Entity\Type', null),
				'registrationNumber' => $this->reqNestedParam('params', 'vehicle_reg', null),
				'fuelType' => $this->reqNestedParamRef('params', 'vehicle_fuel_type', 'Stock\Entity\FuelType', null),
				'transmissionType' => $this->reqNestedParamRef('params', 'vehicle_transmission_type', 'Stock\Entity\TransmissionType', null),

				'km' => $this->reqNestedParam('params', 'vehicle_mileage', null),
				'tradePrice' => $this->reqNestedParam('params', 'trade_price', null),
				'retailPrice' => $this->reqNestedParam('params', 'retail_price', null),
                'listPrice' => $this->reqNestedParam('params', 'list_price', null),
				'condition' => $this->reqNestedParamRef('params', 'vehicle_condition', 'Stock\Entity\Condition', null),
				'exteriorColour' => $this->reqNestedParamRef('params', 'vehicle_exterior_colour', 'Stock\Entity\ExteriorColour', null),
				'interiorColour' => $this->reqNestedParamRef('params', 'vehicle_interior_colour', 'Stock\Entity\InteriorColour', null),
				'upholstery' => $this->reqNestedParamRef('params', 'vehicle_upholstery', 'Stock\Entity\Upholstery', null),
				'vinNumber' => $this->reqNestedParam('params', 'vehicle_vin', null),
				'engineNumber' => $this->reqNestedParam('params', 'vehicle_engine_number', null),
				'previousRepairsNoted' => $this->reqNestedParam('params', 'vehicle_prev_repairs', null),
				'previousRepairsNotes' => $this->reqNestedParam('params', 'vehicle_prev_repairs_comments', null),
				'accessories' => $accessories,
				'accessoryNotes' => $this->reqNestedParam('params', 'vehicle_accessories_comments', null),
				'damages' => $this->reqNestedParam('params', 'vehicle_damages', null),
				'damageNotes' => $this->reqNestedParam('params', 'vehicle_damages_comments', null),
				'spareKeys' => $keys,
				'papers' => $this->reqNestedParamRef('params', 'vehicle_papers', 'Stock\Entity\Paper', null),
				'fullServiceHistory' => $this->reqNestedParamRef('params', 'vehicle_fsh', 'Stock\Entity\FullServiceHistory', null),
				'fshNotes' => $this->reqNestedParam('params', 'vehicle_fsh_notes', null),
		);
		$valuationData = array(
				'firstName' => $this->reqNestedParam('params', 'customer_name', null),
				'familyName' => $this->reqNestedParam('params', 'customer_surname', null),
				'mobile' => $this->reqNestedParam('params', 'customer_mobile_number', null),
				'department' => $this->reqNestedParam('params', 'customer_department', null),
				'idNumber' => $this->reqNestedParam('params', 'customer_id_number', null),
				'email' => $this->reqNestedParam('params', 'customer_email', null)
		);
		$emailValidator = new \Zend\Validator\EmailAddress();
			if (!$emailValidator->isValid($valuationData['email']))
			{
				$valuationData['email'] = null;
			}
		try
		{
			$wValuation = $this->serviceLocator->get('Valuation');
			$valuation->fromArray($valuationData);
			$stock->fromArray($stockData);
			$stock->uvi = $uvi;
			$stock->jobState = 'Valuation';
			if ($isNewEntry)
			{
				$this->em->persist($stock);
				$this->em->persist($valuation);
			}
			$this->em->flush($stock);
			$this->em->flush($valuation);
			if ($isNewEntry)
			{
				$stock->valuation = $valuation;
				$stock->postInsert();
				$valuation->postInsert();
				$this->em->flush($stock);
			}
			$oValuation = new \Valuation\Service\Valuation();
			$oValuation->setWorkflow($wValuation);
			$wValuation->loadJob($valuation->id);
			if (!$pgOverrideDoNotSend)
			{
				$wValuation->changeState('This.PendingValuation');
			}
			if (!$pgOverrideDoNotSend && $this->reqNestedParam('params', 'send_to_priceguide', false))
			{
				$authData = \Utility\Registry::getAuthData();
				$clubs = $this->em->getRepository('\PriceGuide\Entity\Club')
					->findBy(array(
							'company' => $this->em->getReference('\Company\Entity\Company', $authData['company']['id']),
							'useAsDefault' => true
					));
				if (!empty($clubs))
				{
					$sendTo = array();
					foreach ($clubs as $club)
					{
						$sendTo[] = array('id' => $club->id);
					}
					$input = new \Workspace\Utility\ServiceInput('ParamSet', array(
							'id' => $valuation->id,
							'Context' => array(
									'clubs' => $sendTo
							),
							'Valuation' => array(
									'firstName'     => $valuationData['firstName'],
									'familyName'    => $valuationData['familyName'],
									'mobile'        => $valuationData['mobile']
							),
							'Stock' => array(
									'vehicleYear'          => $stockData['vehicleYear']->id,
									'type'                 => $stockData['type']->id,
									'registrationNumber'   => $stockData['registrationNumber'],
									'fuelType'             => $stockData['fuelType']->id,
									'transmissionType'     => $stockData['transmissionType']->id
							)));
					$serviceInput = new \Workspace\Utility\ServiceInput('ServiceInput', array(
							'data' => $input->pack()
					));
					$contract = $serviceInput->pack();
					$result = $wValuation->executeRoute('Valuation.SendToPriceGuide', $valuation->id, $serviceInput->pack());
				}
			}
		}
		catch (\Exception $e)
		{
			\Utility\Debug::errorLog('SynchValuationInfo Exception', "$e");
			return array(
					'Meta' => array(
							'Task'         => 'SynchValuationInfo',
							'Status'       => 'Exception',
							'StatusReason' => 'Server code exception occurred. Could not synchronize data.'
					),
					'Data' => array('uvi' => $uvi)
			);
		}
		unset($this->request['params']);
		if (!$valuation->id)
		{
			return array(
						'Meta' => array(
								'Task'         => 'SynchValuationInfo',
								'Status'       => 'Error',
								'StatusReason' => 'Could not save entry to database.'
						),
						'Data' => array('uvi' => $uvi)
				);
		}
		else
		{
			return array(
						'Meta' => array(
								'Task'         => 'SynchValuationInfo',
								'Status'       => 'Success',
								'StatusReason' => 'Synchronized.'
						),
						'Data' => array('uvi' => $uvi)
				);
		}
	}

	/**
	 * Retrieve a dataset.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function getDatasetAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'GetDataset',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		if (!isset($this->request['datasetName']))
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'GetDataset',
							'Status'       => 'Fail',
							'StatusReason' => 'Api call requires valid `datasetName` parameter.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}
		$ds = $this->request['datasetName'];
		$reverse = false;
		switch ($ds)
		{
			#-> Location
			case 'countries': $entityName = '\Location\Entity\Country'; break;
			case 'regions': $entityName = '\Location\Entity\Region'; break;
			case 'towns': $entityName = '\Location\Entity\Town'; break;
			#-> Stock
			case 'valuations': $entityName = '\Valuation\Entity\Valuation'; break;
			case 'accessories': $entityName = '\Stock\Entity\Accessory'; break;
			case 'damages': $entityName = '\Stock\Entity\Damage'; break;
			case 'upholstery': $entityName = '\Stock\Entity\Upholstery'; break;
			case 'condition': $entityName = '\Stock\Entity\Condition'; break;
			case 'exteriorColours': $entityName = '\Stock\Entity\ExteriorColour'; break;
			case 'interiorColours': $entityName = '\Stock\Entity\InteriorColour'; break;
			case 'fuelTypes': $entityName = '\Stock\Entity\FuelType'; break;
			case 'fullServiceHistory': $entityName = '\Stock\Entity\FullServiceHistory'; break;
			case 'transmissionTypes': $entityName = '\Stock\Entity\TransmissionType'; break;
			case 'natis': $entityName = '\Stock\Entity\Natis'; break;
			case 'papers': $entityName = '\Stock\Entity\Paper'; break;
			case 'years': $entityName = '\Stock\Entity\Year'; $reverse = true; break;
			case 'categories': $entityName = '\Stock\Entity\Category'; break;
			case 'makes': $entityName = '\Stock\Entity\Make'; break;
			case 'models': $entityName = '\Stock\Entity\Model'; break;
			case 'types': $entityName = '\Stock\Entity\Type'; break;
			default:
				return new \Zend\View\Model\JsonModel(array(
						'Meta' => array(
								'Task'         => 'GetDataset',
								'Status'       => 'Fail',
								'StatusReason' => 'Unknown dataset requested.',
								'DatasetName'  => $ds
						),
						'Data' => array()
				));
				break;
		}
		$em = $this->getServiceLocator()->get('doctrine.entitymanager.orm_default');
		if ('Build' == $entityName::PULL_SYNCH_STRATEGY || 'years' == $ds || 'accessories' == $ds)
		{
			$filter = ' WHERE 1 = 1 ';
			$filter .= defined($entityName . '::JOB_QUEUE')
				? ' AND a.queueStatus = 1 '
				: '';
			$filter .= $entityName::ARCHIVE
				? ' AND a.archived = 0 '
				: '';
			if (method_exists($entityName, 'getSynchQuery'))
			{
				$query = $em->createQuery(
						$entityName::getSynchQuery()
						);
			}
			else
			{
				$query = $em->createQuery(
						"SELECT a FROM $entityName a $filter ORDER BY a.id ASC"
				);
			}
			$data = $query->getResult();
			if (defined($entityName . '::JOB_QUEUE'))
			{
				foreach ($data as $rowId => $record)
				{
					$data[$rowId] = $record->toQueueArray();
				}
			}
			else
			{
				foreach ($data as $rowId => $record)
				{
					$data[$rowId] = $record->toSynchArray();
				}
			}
			if ($reverse)
			{
				$data = array_reverse($data);
			}

		}
		else
		{
			$data = array();
			$version = isset($this->request['version'])
				? (int) $this->request['version']
				: 1;
			$filter = '';
			$filter .= defined($entityName . '::JOB_QUEUE')
				? ' AND a.queueStatus = 1'
				: '';
			$filter .= $entityName::ARCHIVE
				? ' AND a.archived = 0'
				: '';
			if (method_exists($entityName, 'getSynchFilter'))
			{
				$synchMod = $entityName::getSynchFilter();
			}
			else
			{
				$synchMod = array(
						'Join' => '',
						'Filter' => ''
				);
			}

			#-> Records to add.
			$query = "SELECT [SELECTION] "
					. "FROM $entityName a "
					. $synchMod['Join']
					. "WHERE a.archived = :archived"
					. " $filter " . $synchMod['Filter']
					. " AND a.createVersion = :version"
					. " ORDER BY a.id ASC";
			$params = array(
					'archived' => false,
					'version'  => $version
			);
			$data['Create'] = $em->createQuery(
					str_replace('[SELECTION]', 'a', $query)
					)
				->setParameters($params)
				->getResult();

			#-> Records to update.
			$query = "SELECT [SELECTION] "
					. "FROM $entityName a "
					. $synchMod['Join']
					. "WHERE a.archived = :archived"
					. " $filter " . $synchMod['Filter']
					. " AND a.createVersion != :version"
					. " AND a.updateVersion = :version"
					. " ORDER BY a.id ASC";
			$params = array(
					'archived'  => false,
					'version'  => $version
			);
			$data['Update'] = $em->createQuery(
					str_replace('[SELECTION]', 'a', $query)
					)
				->setParameters($params)
				->getResult();
			$data['Delete'] = array();


			#-> Finalize.
			if (defined($entityName . '::JOB_QUEUE'))
			{
				foreach ($data as $type => $dataset)
				{
					foreach ($dataset as $rowId => $record)
					{
						$data[$type][$rowId] = $record->toQueueArray();
					}
				}
			}
			else
			{
				foreach ($data as $type => $dataset)
				{
					foreach ($dataset as $rowId => $record)
					{
						$data[$type][$rowId] = $record->toSynchArray();
					}
				}
			}
		}

		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
					'Task'         => 'GetDataset',
					'Status'       => 'Success',
					'StatusReason' => '',
					'DatasetName'  => $ds,
					'Strategy'     => $entityName::PULL_SYNCH_STRATEGY,
					'DataPages'    => 1,
					'Page'         => 1,
					'StartTime'    => 0,
					'EndTime'      => date('Y-m-d H:i:s')

				),
				'Data' => $data
		));
	}

	/**
	 * Retrieve a dataset.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function getVersionedDatasetAction()
	{
		if (!$this->initPost())
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'GetDataset',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		if (!isset($this->request['datasetName']))
		{
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'GetDataset',
							'Status'       => 'Fail',
							'StatusReason' => 'Api call requires valid `datasetName` parameter.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}
		$ds = $this->request['datasetName'];
		switch ($ds)
		{
			#-> Location
			case 'countries': $entityName = '\Location\Entity\Country'; break;
			case 'regions': $entityName = '\Location\Entity\Region'; break;
			case 'towns': $entityName = '\Location\Entity\Town'; break;
			#-> Stock
			case 'valuations': $entityName = '\Valuation\Entity\Valuation'; break;
			case 'accessories': $entityName = '\Stock\Entity\Accessory'; break;
			case 'damages': $entityName = '\Stock\Entity\Damage'; break;
			case 'upholstery': $entityName = '\Stock\Entity\Upholstery'; break;
			case 'condition': $entityName = '\Stock\Entity\Condition'; break;
			case 'exteriorColours': $entityName = '\Stock\Entity\ExteriorColour'; break;
			case 'interiorColours': $entityName = '\Stock\Entity\InteriorColour'; break;
			case 'fuelTypes': $entityName = '\Stock\Entity\FuelType'; break;
			case 'fullServiceHistory': $entityName = '\Stock\Entity\FullServiceHistory'; break;
			case 'transmissionTypes': $entityName = '\Stock\Entity\TransmissionType'; break;
			case 'natis': $entityName = '\Stock\Entity\Natis'; break;
			case 'papers': $entityName = '\Stock\Entity\Paper'; break;
			case 'years': $entityName = '\Stock\Entity\Year'; break;
			case 'categories': $entityName = '\Stock\Entity\Category'; break;
			case 'makes': $entityName = '\Stock\Entity\Make'; break;
			case 'models': $entityName = '\Stock\Entity\Model'; break;
			case 'types': $entityName = '\Stock\Entity\Type'; break;
			default:
				return new \Zend\View\Model\JsonModel(array(
						'Meta' => array(
								'Task'         => 'GetDataset',
								'Status'       => 'Fail',
								'StatusReason' => 'Unknown dataset requested.',
								'DatasetName'  => $ds
						),
						'Data' => array()
				));
				break;
		}
		$em = $this->getServiceLocator()->get('doctrine.entitymanager.orm_default');
		if ('Build' == $entityName::PULL_SYNCH_STRATEGY)
		{
			/* $timeStart = 0;
			$timeEnd   = time();
			$numPages  = 1;
			$page      = 1;
			$timestamp = time(); */
			$filter = ' WHERE 1 = 1 ';
			$filter .= defined($entityName . '::JOB_QUEUE')
				? ' AND a.queueStatus = 1 '
				: '';
			$filter .= $entityName::ARCHIVE
				? ' AND a.archived = 0 '
				: '';
			if (method_exists($entityName, 'getSynchQuery'))
			{
				$query = $em->createQuery(
						$entityName::getSynchQuery()
						);
			}
			else
			{
				$query = $em->createQuery(
						"SELECT a FROM $entityName a $filter ORDER BY a.id ASC"
				);
			}

			$data = $query->getResult();
			if (defined($entityName . '::JOB_QUEUE'))
			{
				foreach ($data as $rowId => $record)
				{
					$data[$rowId] = $record->toQueueArray();
				}
			}
			else
			{
				foreach ($data as $rowId => $record)
				{
					$data[$rowId] = $record->toSynchArray();
				}
			}
		}
		else
		{
			$data = array();
			/* $page = isset($this->request['page'])
				? (int) $this->request['page']
				: 1;
			$timeStart = isset($this->request['timeStart'])
				? (int) $this->request['timeStart']
				: 0;
			$timeEnd = isset($this->request['timeEnd'])
						&& 0 != $this->request['timeEnd']
				? (int) $this->request['timeEnd']
				: time();
			$recs = 1000; */
			$version = isset($this->request['version'])
				? (int) $this->request['version']
				: 0;
			$version++;
			$filter = '';
			$filter .= defined($entityName . '::JOB_QUEUE')
				? ' AND a.queueStatus = 1'
				: '';
			$filter .= $entityName::ARCHIVE
				? ' AND a.archived = 0'
				: '';
			if (method_exists($entityName, 'getSynchFilter'))
			{
				$synchMod = $entityName::getSynchFilter();
			}
			else
			{
				$synchMod = array(
						'Join' => '',
						'Filter' => ''
				);
			}
			//$dateTimeFormat = \Utility\Definitions\Locale::getDateTimeFormat();
			//$numRecs = 0;

			#-> Records to add.
			$query = "SELECT [SELECTION] "
					. "FROM $entityName a "
					. $synchMod['Join']
					. "WHERE a.archived = :archived"
					. " $filter " . $synchMod['Filter']
					. " AND a.createVersion = :version"
					. " ORDER BY a.id ASC";
			$params = array(
					'archived' => false,
					'version'  => $version
					/* 'timeStart' => date($dateTimeFormat, $timeStart),
					'timeEnd'   => date($dateTimeFormat, $timeEnd) */
			);
			/* $numRecsRes = $em->createQuery(
					str_replace('[SELECTION]', 'COUNT(a.id) AS total', $query)
					)
				->setParameters($params)
				->getSingleResult();
			$numRecs = ($numRecsRes['total'] > $numRecs)
				? $numRecsRes['total']
				: $numRecs; */
			$data['Create'] = $em->createQuery(
					str_replace('[SELECTION]', 'a', $query)
					)
				->setParameters($params)
				->getResult();
				/* ->setFirstResult(($page -1) * $recs)
				->setMaxResults($recs) */

			#-> Records to update.
			$query = "SELECT [SELECTION] "
					. "FROM $entityName a "
					. $synchMod['Join']
					. "WHERE a.archived = :archived"
					. " $filter " . $synchMod['Filter']
					. " AND a.createVersion != :version"
					. " AND a.updateVersion = :version"
					. " ORDER BY a.id ASC";
			$params = array(
					'archived'  => false,
					'version'  => $version
					/* 'timeStart' => date($dateTimeFormat, $timeStart),
					'timeEnd'   => date($dateTimeFormat, $timeEnd) */
			);
			/* $numRecsRes = $em->createQuery(
					str_replace('[SELECTION]', 'COUNT(a.id) AS total', $query)
					)
				->setParameters($params)
				->getSingleResult();
			$numRecs = ($numRecsRes['total'] > $numRecs)
				? $numRecsRes['total']
				: $numRecs; */
			$data['Update'] = $em->createQuery(
					str_replace('[SELECTION]', 'a', $query)
					)
				->setParameters($params)
				->getResult();
				/* ->setFirstResult(($page -1) * $recs)
				->setMaxResults($recs) */

			#-> Records to remove.
			/* $query = "SELECT [SELECTION] "
					. "FROM $entityName a "
					. $synchMod['Join']
					. "WHERE a.archived = :archived"
					. " $filter " . $synchMod['Filter']
					. " AND a.created NOT BETWEEN :timeStart AND :timeEnd"
					. " AND a.updated BETWEEN :timeStart AND :timeEnd"
					. " ORDER BY a.id ASC";
			$params = array(
					'archived'  => true,
					'timeStart' => date($dateTimeFormat, $timeStart),
					'timeEnd'   => date($dateTimeFormat, $timeEnd)
			);
			$numRecsRes = $em->createQuery(
					str_replace('[SELECTION]', 'COUNT(a.id) AS total', $query)
					)
				->setParameters($params)
				->getSingleResult();
			$numRecs = ($numRecsRes['total'] > $numRecs)
				? $numRecsRes['total']
				: $numRecs;
			$data['Delete'] = $em->createQuery(
					str_replace('[SELECTION]', 'a', $query)
					)
				->setParameters($params)
				->setFirstResult(($page -1) * $recs)
				->setMaxResults($recs)
				->getResult(); */
			$data['Delete'] = array();
			if (empty($data['Create'])
				&& empty($data['Update'])
				&& empty($data['Delete']))
			{
				$version--;
			}


			#-> Finalize.
			if (defined($entityName . '::JOB_QUEUE'))
			{
				foreach ($data as $type => $dataset)
				{
					foreach ($dataset as $rowId => $record)
					{
						$data[$type][$rowId] = $record->toQueueArray();
					}
				}
			}
			else
			{
				foreach ($data as $type => $dataset)
				{
					foreach ($dataset as $rowId => $record)
					{
						$data[$type][$rowId] = $record->toSynchArray();
					}
				}
			}
		}

		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array(
					'Task'         => 'GetDataset',
					'Status'       => 'Success',
					'StatusReason' => '',
					'DatasetName'  => $ds,
					'Strategy'     => $entityName::PULL_SYNCH_STRATEGY,
					'Version'      => $version

				),
				'Data' => $data
		));
	}

	/**
	 * Retrieve trade and retail from mmCode.
	 * @return \Zend\View\Model\JsonModel
	 */
	public function transunionFromMmcodeAction()
	{
		if (!$this->initPost())
		{
			\Utility\Debug::errorLog('transunionFromMmcodeAction.Error', 'NO AUTH');
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'TransunionFromMmcodeAction',
							'Status'       => 'Exception',
							'StatusReason' => 'Invalid session requested.',
							'DatasetName'  => null
					),
					'Data' => array()
			));
		}

		if (!isset($this->request['mmCode']) || !is_numeric($this->request['mmCode'])
			|| !isset($this->request['vehicleYear']) || !is_numeric($this->request['vehicleYear']))
		{
			\Utility\Debug::errorLog('transunionFromMmcodeAction.Error', 'missing mmCode or vehicleYear');
			return new \Zend\View\Model\JsonModel(array(
					'Meta' => array(
							'Task'         => 'TransunionFromMmcodeAction',
							'Status'       => 'Fail',
							'StatusReason' => 'Requires `mmCode` and `vehicleYear`. Optionally accepts `mileage` and `condition`.'
					),
					'Data' => array()
			));
		}
		$input = new \Workspace\Utility\ServiceInput('ParamSet', array(
				'Filter' => array(
						'mmCode'      => $this->request['mmCode'],
						'vehicleYear' => $this->request['vehicleYear'],
						'mileage'     => isset($this->request['mileage']) && !empty($this->request['mileage'])
											? $this->request['mileage']
											: null,
						'condition'   => isset($this->request['condition']) && !empty($this->request['condition'])
											? $this->request['condition']
											: null
				)
		));
		$serviceInput = new \Workspace\Utility\ServiceInput('ServiceInput', array(
				'data' => $input->pack()
		));
		$stockUtilService = $this->serviceLocator->get('Stock.Service.Utility');
		$response = $stockUtilService->deviceTransUnionFromMmCode($serviceInput->pack());
		return new \Zend\View\Model\JsonModel(array(
				'Meta' => array_merge(array('Task' => 'Login'), $response['Meta']),
				'Data' => $response['Data']
		));
	}

}